@@ -18,7 +18,7 @@ Plot::Plot( QWidget *parent ):
     setCanvasColor( Qt::darkCyan );
 
     QwtPlotGrid *grid = new QwtPlotGrid;
-    grid->setMajPen( QPen( Qt::white, 0, Qt::DotLine ) );
+    grid->setMajorPen( Qt::white, 0, Qt::DotLine );
     grid->attach( this );
 
     // axes
@@ -100,7 +100,7 @@ bool Plot::eventFilter( QObject *object, QEvent *e )
 {
     if ( e->type() == QEvent::Resize )
     {
-        const QSize size = ( ( QResizeEvent * )e )->size();
+        const QSize size = static_cast<QResizeEvent *>( e )->size();
         if ( object == axisWidget( yLeft ) )
         {
             const QwtScaleWidget *scaleWidget = axisWidget( yLeft );
@@ -139,7 +139,7 @@ void Plot::insertCurve( int axis, double base )
     else
         o = Qt::Vertical;
 
-    QRgb rgb = ( uint )rand();
+    QRgb rgb = static_cast<QRgb>( rand() );
     insertCurve( o, QColor( rgb ), base );
     replot();
 }
