@@ -29,9 +29,8 @@ MainWindow::MainWindow():
 
 void MainWindow::resizeEvent( QResizeEvent * )
 {
-#ifdef Q_WS_X11
+    // Qt 4.7.1: QGradient::StretchToDeviceMode is buggy on X11
     updateGradient();
-#endif
 }
 
 void MainWindow::updateGradient()
@@ -41,15 +40,7 @@ void MainWindow::updateGradient()
     const QColor buttonColor = pal.color( QPalette::Button );
     const QColor midLightColor = pal.color( QPalette::Midlight );
 
-#ifdef Q_WS_X11
-    // Qt 4.7.1: QGradient::StretchToDeviceMode is buggy on X11
-
     QLinearGradient gradient( rect().topLeft(), rect().topRight() );
-#else
-    QLinearGradient gradient( 0, 0, 1, 0 );
-    gradient.setCoordinateMode( QGradient::StretchToDeviceMode );
-#endif
-
     gradient.setColorAt( 0.0, midLightColor );
     gradient.setColorAt( 0.7, buttonColor );
     gradient.setColorAt( 1.0, buttonColor );
