@@ -3,6 +3,7 @@
 #include <qwt_plot_curve.h>
 #include <qwt_symbol.h>
 #include <qwt_plot_directpainter.h>
+#include <qwt_painter.h>
 #include "incrementalplot.h"
 #include <qpaintengine.h>
 
@@ -40,17 +41,17 @@ IncrementalPlot::IncrementalPlot( QWidget *parent ):
 {
     d_directPainter = new QwtPlotDirectPainter( this );
 
-#if defined(Q_WS_X11)
-    canvas()->setAttribute( Qt::WA_PaintOutsidePaintEvent, true );
-    canvas()->setAttribute( Qt::WA_PaintOnScreen, true );
+    if ( QwtPainter::isX11GraphicsSystem() )
+    {
+#if QT_VERSION < 0x050000
+        canvas()->setAttribute( Qt::WA_PaintOutsidePaintEvent, true );
 #endif
+        canvas()->setAttribute( Qt::WA_PaintOnScreen, true );
+    }
 
     d_curve = new QwtPlotCurve( "Test Curve" );
-    d_curve->setStyle( QwtPlotCurve::NoCurve );
     d_curve->setData( new CurveData() );
-
-    d_curve->setSymbol( new QwtSymbol( QwtSymbol::XCross,
-        Qt::NoBrush, QPen( Qt::white ), QSize( 4, 4 ) ) );
+    showSymbols( true );
 
     d_curve->attach( this );
 
@@ -64,8 +65,8 @@ IncrementalPlot::~IncrementalPlot()
 
 void IncrementalPlot::appendPoint( const QPointF &point )
 {
-    CurveData *data = static_cast<CurveData *>( d_curve->data() );
-    data->append( point );
+    CurveData *curveData = static_cast<CurveData *>( d_curve->data() );
+    curveData->append( point );
 
     const bool doClip = !canvas()->testAttribute( Qt::WA_PaintOnScreen );
     if ( doClip )
@@ -93,13 +94,31 @@ void IncrementalPlot::appendPoint( const QPointF &point )
     }
 
     d_directPainter->drawSeries( d_curve,
-        data->size() - 1, data->size() - 1 );
+        curveData->size() - 1, curveData->size() - 1 );
 }
 
 void IncrementalPlot::clearPoints()
 {
-    CurveData *data = static_cast<CurveData *>( d_curve->data() );
-    data->clear();
+    CurveData *curveData = static_cast<CurveData *>( d_curve->data() );
+    curveData->clear();
+
+    replot();
+}
+
+void IncrementalPlot::showSymbols( bool on )
+{
+    if ( on )
+    {
+        d_curve->setStyle( QwtPlotCurve::NoCurve );
+        d_curve->setSymbol( new QwtSymbol( QwtSymbol::XCross,
+            Qt::NoBrush, QPen( Qt::white ), QSize( 4, 4 ) ) );
+    }
+    else
+    {
+        d_curve->setPen( Qt::white );
+        d_curve->setStyle( QwtPlotCurve::Dots );
+        d_curve->setSymbol( NULL );
+    }
 
     replot();
 }
