@@ -8,32 +8,68 @@
  *****************************************************************************/
 
 #include "qwt_analog_clock.h"
+#include "qwt_round_scale_draw.h"
 #include <qmath.h>
 #include <qlocale.h>
 
+class QwtAnalogClockScaleDraw: public QwtRoundScaleDraw
+{
+public:
+    QwtAnalogClockScaleDraw()
+    {
+        setSpacing( 8 );
+
+        enableComponent( QwtAbstractScaleDraw::Backbone, false );
+
+        setTickLength( QwtScaleDiv::MinorTick, 2 );
+        setTickLength( QwtScaleDiv::MediumTick, 4 );
+        setTickLength( QwtScaleDiv::MajorTick, 8 );
+
+        setPenWidth( 1 );
+    }
+
+    virtual QwtText label( double value ) const
+    {
+        if ( qFuzzyCompare( value + 1.0, 1.0 ) )
+            value = 60.0 * 60.0 * 12.0;
+
+        return QLocale().toString( qRound( value / ( 60.0 * 60.0 ) ) );
+    }
+};
+
 /*!
   Constructor
   \param parent Parent widget
 */
 QwtAnalogClock::QwtAnalogClock( QWidget *parent ):
     QwtDial( parent )
-{
-    initClock();
-}
-
-void QwtAnalogClock::initClock()
 {
     setWrapping( true );
     setReadOnly( true );
 
     setOrigin( 270.0 );
-    setRange( 0.0, 60.0 * 60.0 * 12.0 ); // seconds
-    setScale( -1, 5, 60.0 * 60.0 );
+    setScaleDraw( new QwtAnalogClockScaleDraw() );
 
-    setScaleComponents( 
-        QwtAbstractScaleDraw::Ticks | QwtAbstractScaleDraw::Labels );
-    setScaleTicks( 1, 0, 8 );
-    scaleDraw()->setSpacing( 8 );
+    setTotalSteps( 60 );
+
+    const int secondsPerHour = 60.0 * 60.0;
+
+    QList<double> majorTicks;
+    QList<double> minorTicks;
+
+    for ( int i = 0; i < 12; i++ )
+    {
+        majorTicks += i * secondsPerHour;
+
+        for ( int j = 1; j < 5; j++ )
+            minorTicks += i * secondsPerHour + j * secondsPerHour / 5.0;
+    }
+
+    QwtScaleDiv scaleDiv;
+    scaleDiv.setInterval( 0.0, 12.0 * secondsPerHour );
+    scaleDiv.setTicks( QwtScaleDiv::MajorTick, majorTicks );
+    scaleDiv.setTicks( QwtScaleDiv::MinorTick, minorTicks );
+    setScale( scaleDiv );
 
     QColor knobColor = palette().color( QPalette::Active, QPalette::Text );
     knobColor = knobColor.dark( 120 );
@@ -59,7 +95,7 @@ void QwtAnalogClock::initClock()
         hand->setWidth( width );
 
         d_hand[i] = NULL;
-        setHand( ( Hand )i, hand );
+        setHand( static_cast<Hand>( i ), hand );
     }
 }
 
@@ -71,7 +107,7 @@ QwtAnalogClock::~QwtAnalogClock()
 }
 
 /*!
-  Nop method, use setHand instead
+  Nop method, use setHand() instead
   \sa setHand()
 */
 void QwtAnalogClock::setNeedle( QwtDialNeedle * )
@@ -81,7 +117,7 @@ void QwtAnalogClock::setNeedle( QwtDialNeedle * )
 }
 
 /*!
-   Set a clockhand
+   Set a clock hand
    \param hand Specifies the type of hand
    \param needle Hand
    \sa hand()
@@ -120,9 +156,6 @@ const QwtDialNeedle *QwtAnalogClock::hand( Hand hd ) const
 
 /*!
   \brief Set the current time
-
-  This is the same as QwtAnalogClock::setTime(), but Qt < 3.0
-  can't handle default parameters for slots.
 */
 void QwtAnalogClock::setCurrentTime()
 {
@@ -144,24 +177,10 @@ void QwtAnalogClock::setTime( const QTime &time )
         setValid( false );
 }
 
-/*!
-  Find the scale label for a given value
-
-  \param value Value
-  \return Label
-*/
-QwtText QwtAnalogClock::scaleLabel( double value ) const
-{
-    if ( qFuzzyCompare( value + 1.0, 1.0 ) )
-        value = 60.0 * 60.0 * 12.0;
-
-    return QLocale().toString( qRound( value / ( 60.0 * 60.0 ) ) );
-}
-
 /*!
   \brief Draw the needle
 
-  A clock has no single needle but three hands instead. drawNeedle
+  A clock has no single needle but three hands instead. drawNeedle()
   translates value() into directions for the hands and calls
   drawHand().
 
@@ -181,7 +200,7 @@ void QwtAnalogClock::drawNeedle( QPainter *painter, const QPointF &center,
     if ( isValid() )
     {
         const double hours = value() / ( 60.0 * 60.0 );
-        const double minutes = 
+        const double minutes =
             ( value() - qFloor(hours) * 60.0 * 60.0 ) / 60.0;
         const double seconds = value() - qFloor(hours) * 60.0 * 60.0
             - qFloor(minutes) * 60.0;
@@ -193,13 +212,9 @@ void QwtAnalogClock::drawNeedle( QPainter *painter, const QPointF &center,
 
         for ( int hand = 0; hand < NHands; hand++ )
         {
-            double d = angle[hand];
-            if ( direction() == Clockwise )
-                d = 360.0 - d;
-
-            d -= origin();
-
-            drawHand( painter, ( Hand )hand, center, radius, d, colorGroup );
+            const double d = 360.0 - angle[hand] - origin();
+            drawHand( painter, static_cast<Hand>( hand ),
+                center, radius, d, colorGroup );
         }
     }
 }
