@@ -13,7 +13,7 @@
 #include <qdatetime.h>
 #include <limits.h>
 
-static inline double qwtMsecsForType( QwtDate::IntervalType type )
+static inline double qwtMsecsForType( int type )
 {
     static const double msecs[] =
     {
@@ -43,7 +43,7 @@ static inline int qwtAlignValue(
 }
 
 static double qwtIntervalWidth( const QDateTime &minDate,
-    const QDateTime &maxDate, QwtDate::IntervalType intervalType ) 
+    const QDateTime &maxDate, QwtDate::IntervalType intervalType )
 {
     switch( intervalType )
     {
@@ -79,7 +79,7 @@ static double qwtIntervalWidth( const QDateTime &minDate,
         }
         case QwtDate::Month:
         {
-            const double years = 
+            const double years =
                 double( maxDate.date().year() ) - minDate.date().year();
 
             int months = maxDate.date().month() - minDate.date().month();
@@ -90,7 +90,7 @@ static double qwtIntervalWidth( const QDateTime &minDate,
         }
         case QwtDate::Year:
         {
-            double years = 
+            double years =
                 double( maxDate.date().year() ) - minDate.date().year();
 
             if ( maxDate.date().month() < minDate.date().month() )
@@ -103,9 +103,9 @@ static double qwtIntervalWidth( const QDateTime &minDate,
     return 0.0;
 }
 
-static double qwtRoundedIntervalWidth( 
-    const QDateTime &minDate, const QDateTime &maxDate, 
-    QwtDate::IntervalType intervalType ) 
+static double qwtRoundedIntervalWidth(
+    const QDateTime &minDate, const QDateTime &maxDate,
+    QwtDate::IntervalType intervalType )
 {
     const QDateTime minD = QwtDate::floor( minDate, intervalType );
     const QDateTime maxD = QwtDate::ceil( maxDate, intervalType );
@@ -130,7 +130,7 @@ static inline int qwtStepCount( int intervalSize, int maxSteps,
     return 0;
 }
 
-static int qwtStepSize( int intervalSize, int maxSteps, uint base ) 
+static int qwtStepSize( int intervalSize, int maxSteps, uint base )
 {
     if ( maxSteps <= 0 )
         return 0;
@@ -161,7 +161,7 @@ static int qwtStepSize( int intervalSize, int maxSteps, uint base )
     return 0;
 }
 
-static int qwtDivideInterval( double intervalSize, int numSteps, 
+static int qwtDivideInterval( double intervalSize, int numSteps,
     const int limits[], size_t numLimits )
 {
     const int v = qCeil( intervalSize / double( numSteps ) );
@@ -180,7 +180,7 @@ static double qwtDivideScale( double intervalSize, int numSteps,
 {
     if ( intervalType != QwtDate::Day )
     {
-        if ( ( intervalSize > numSteps ) && 
+        if ( ( intervalSize > numSteps ) &&
             ( intervalSize <= 2 * numSteps ) )
         {
             return 2.0;
@@ -195,7 +195,7 @@ static double qwtDivideScale( double intervalSize, int numSteps,
         case QwtDate::Minute:
         {
             static int limits[] = { 1, 2, 5, 10, 15, 20, 30, 60 };
-    
+
             stepSize = qwtDivideInterval( intervalSize, numSteps,
                 limits, sizeof( limits ) / sizeof( int ) );
 
@@ -204,7 +204,7 @@ static double qwtDivideScale( double intervalSize, int numSteps,
         case QwtDate::Hour:
         {
             static int limits[] = { 1, 2, 3, 4, 6, 12, 24 };
-    
+
             stepSize = qwtDivideInterval( intervalSize, numSteps,
                 limits, sizeof( limits ) / sizeof( int ) );
 
@@ -273,13 +273,13 @@ static double qwtDivideMajorStep( double stepSize, int maxMinSteps,
 
             if ( stepSize > maxMinSteps )
             {
-                numSteps = qwtStepCount( stepSize, maxMinSteps, 
+                numSteps = qwtStepCount( stepSize, maxMinSteps,
                     limits, sizeof( limits ) / sizeof( int ) );
 
             }
             else
             {
-                numSteps = qwtStepCount( stepSize * 60, maxMinSteps, 
+                numSteps = qwtStepCount( stepSize * 60, maxMinSteps,
                     limits, sizeof( limits ) / sizeof( int ) );
             }
 
@@ -358,7 +358,7 @@ static double qwtDivideMajorStep( double stepSize, int maxMinSteps,
                 }
                 else
                 {
-                    minStepSize = QwtScaleArithmetic::divideInterval( 
+                    minStepSize = QwtScaleArithmetic::divideInterval(
                         stepSizeInWeeks, maxMinSteps, 10 );
                 }
             }
@@ -400,7 +400,7 @@ static double qwtDivideMajorStep( double stepSize, int maxMinSteps,
                 if ( numSteps > 0 )
                     minStepSize = double( stepSize ) / numSteps;
             }
-                
+
             break;
         }
         default:
@@ -444,17 +444,17 @@ static QList<double> qwtDstTicks( const QDateTime &dateTime,
     return ticks;
 }
 
-static QwtScaleDiv qwtDivideToSeconds( 
+static QwtScaleDiv qwtDivideToSeconds(
     const QDateTime &minDate, const QDateTime &maxDate,
     double stepSize, int maxMinSteps,
-    QwtDate::IntervalType intervalType ) 
+    QwtDate::IntervalType intervalType )
 {
     // calculate the min step size
     double minStepSize = 0;
 
-    if ( maxMinSteps > 1 ) 
+    if ( maxMinSteps > 1 )
     {
-        minStepSize = qwtDivideMajorStep( stepSize, 
+        minStepSize = qwtDivideMajorStep( stepSize,
             maxMinSteps, intervalType );
     }
 
@@ -471,7 +471,7 @@ static QwtScaleDiv qwtDivideToSeconds(
     const double s = qwtMsecsForType( intervalType ) / 1000;
     const int secondsMajor = static_cast<int>( stepSize * s );
     const double secondsMinor = minStepSize * s;
-    
+
     // UTC excludes daylight savings. So from the difference
     // of a date and its UTC counterpart we can find out
     // the daylight saving hours
@@ -483,7 +483,7 @@ static QwtScaleDiv qwtDivideToSeconds(
     QList<double> mediumTicks;
     QList<double> minorTicks;
 
-    for ( QDateTime dt = minDate; dt <= maxDate; 
+    for ( QDateTime dt = minDate; dt <= maxDate;
         dt = dt.addSecs( secondsMajor ) )
     {
         if ( !dt.isValid() )
@@ -500,7 +500,7 @@ static QwtScaleDiv qwtDivideToSeconds(
             {
                 // we add some minor ticks for the DST hour,
                 // otherwise the ticks will be unaligned: 0, 2, 3, 5 ...
-                minorTicks += qwtDstTicks( 
+                minorTicks += qwtDstTicks(
                     dt, secondsMajor, qRound( secondsMinor ) );
             }
 
@@ -516,7 +516,7 @@ static QwtScaleDiv qwtDivideToSeconds(
 
             for ( int i = 1; i < numMinorSteps; i++ )
             {
-                const QDateTime mt = dt.addMSecs( 
+                const QDateTime mt = dt.addMSecs(
                     qRound64( i * secondsMinor * 1000 ) );
 
                 double minorValue = QwtDate::toDouble( mt );
@@ -528,7 +528,7 @@ static QwtScaleDiv qwtDivideToSeconds(
 
                 if ( minorTicks.isEmpty() || minorTicks.last() != minorValue )
                 {
-                    const bool isMedium = ( numMinorSteps % 2 == 0 ) 
+                    const bool isMedium = ( numMinorSteps % 2 == 0 )
                         && ( i != 1 ) && ( i == numMinorSteps / 2 );
 
                     if ( isMedium )
@@ -552,16 +552,16 @@ static QwtScaleDiv qwtDivideToSeconds(
     return scaleDiv;
 }
 
-static QwtScaleDiv qwtDivideToMonths( 
+static QwtScaleDiv qwtDivideToMonths(
     QDateTime &minDate, const QDateTime &maxDate,
-    double stepSize, int maxMinSteps ) 
+    double stepSize, int maxMinSteps )
 {
-    // months are intervals with non 
-    // equidistant ( in ms ) steps: we have to build the 
+    // months are intervals with non
+    // equidistant ( in ms ) steps: we have to build the
     // scale division manually
 
     int minStepDays = 0;
-    int minStepSize = 0.0; 
+    int minStepSize = 0.0;
 
     if ( maxMinSteps > 1 )
     {
@@ -578,7 +578,7 @@ static QwtScaleDiv qwtDivideToMonths(
         }
         else
         {
-            minStepSize = qwtDivideMajorStep( 
+            minStepSize = qwtDivideMajorStep(
                 stepSize, maxMinSteps, QwtDate::Month );
         }
     }
@@ -587,7 +587,7 @@ static QwtScaleDiv qwtDivideToMonths(
     QList<double> mediumTicks;
     QList<double> minorTicks;
 
-    for ( QDateTime dt = minDate; 
+    for ( QDateTime dt = minDate;
         dt <= maxDate; dt = dt.addMonths( stepSize ) )
     {
         if ( !dt.isValid() )
@@ -597,7 +597,7 @@ static QwtScaleDiv qwtDivideToMonths(
 
         if ( minStepDays > 0 )
         {
-            for ( int days = minStepDays; 
+            for ( int days = minStepDays;
                 days < 30; days += minStepDays )
             {
                 const double tick = QwtDate::toDouble( dt.addDays( days ) );
@@ -636,9 +636,9 @@ static QwtScaleDiv qwtDivideToMonths(
     return scaleDiv;
 }
 
-static QwtScaleDiv qwtDivideToYears( 
+static QwtScaleDiv qwtDivideToYears(
     const QDateTime &minDate, const QDateTime &maxDate,
-    double stepSize, int maxMinSteps ) 
+    double stepSize, int maxMinSteps )
 {
     QList<double> majorTicks;
     QList<double> mediumTicks;
@@ -648,7 +648,7 @@ static QwtScaleDiv qwtDivideToYears(
 
     if ( maxMinSteps > 1 )
     {
-        minStepSize = qwtDivideMajorStep( 
+        minStepSize = qwtDivideMajorStep(
             stepSize, maxMinSteps, QwtDate::Year );
     }
 
@@ -701,7 +701,7 @@ static QwtScaleDiv qwtDivideToYears(
         {
             break;
         }
-    }   
+    }
 
     QwtScaleDiv scaleDiv;
     scaleDiv.setInterval( QwtDate::toDouble( minDate ),
@@ -729,13 +729,13 @@ public:
     int utcOffset;
     QwtDate::Week0Type week0Type;
     int maxWeeks;
-};      
+};
 
 
 /*!
   \brief Constructor
 
-  The engine is initialized to build scales for the 
+  The engine is initialized to build scales for the
   given time specification. It classifies intervals > 4 weeks
   as >= Qt::Month. The first week of a year is defined like
   for QwtDate::FirstThursday.
@@ -810,7 +810,7 @@ int QwtDateScaleEngine::utcOffset() const
 
   \sa week0Type(), setMaxWeeks()
   \note week0Type has no effect beside for intervals classified as
-        QwtDate::Week. 
+        QwtDate::Week.
  */
 void QwtDateScaleEngine::setWeek0Type( QwtDate::Week0Type week0Type )
 {
@@ -818,7 +818,7 @@ void QwtDateScaleEngine::setWeek0Type( QwtDate::Week0Type week0Type )
 }
 
 /*!
-  \return Setting how to identify the first week of a year. 
+  \return Setting how to identify the first week of a year.
   \sa setWeek0Type(), maxWeeks()
  */
 QwtDate::Week0Type QwtDateScaleEngine::week0Type() const
@@ -836,7 +836,7 @@ QwtDate::Week0Type QwtDateScaleEngine::week0Type() const
 
   \note In business charts a year is often devided
         into weeks [1-52]
-  \sa maxWeeks(), setWeek0Type() 
+  \sa maxWeeks(), setWeek0Type()
  */
 void QwtDateScaleEngine::setMaxWeeks( int weeks )
 {
@@ -862,8 +862,8 @@ int QwtDateScaleEngine::maxWeeks() const
 
   \return Interval classification
  */
-QwtDate::IntervalType QwtDateScaleEngine::intervalType( 
-    const QDateTime &minDate, const QDateTime &maxDate, 
+QwtDate::IntervalType QwtDateScaleEngine::intervalType(
+    const QDateTime &minDate, const QDateTime &maxDate,
     int maxSteps ) const
 {
     const double jdMin = minDate.date().toJulianDay();
@@ -909,11 +909,11 @@ QwtDate::IntervalType QwtDateScaleEngine::intervalType(
 /*!
   Align and divide an interval
 
-  The algorithm aligns and divides the interval into steps. 
+  The algorithm aligns and divides the interval into steps.
 
   Datetime interval divisions are usually not equidistant and the
   calculated stepSize can only be used as an approximation
-  for the steps calculated by divideScale(). 
+  for the steps calculated by divideScale().
 
   \param maxNumSteps Max. number of steps
   \param x1 First limit of the interval (In/Out)
@@ -950,7 +950,7 @@ void QwtDateScaleEngine::autoScale( int maxNumSteps,
         if ( maxNumSteps < 1 )
             maxNumSteps = 1;
 
-        const QwtDate::IntervalType intvType = 
+        const QwtDate::IntervalType intvType =
             intervalType( from, to, maxNumSteps );
 
         const double width = qwtIntervalWidth( from, to, intvType );
@@ -1014,7 +1014,7 @@ QwtScaleDiv QwtDateScaleEngine::divideScale( double x1, double x2,
         maxMajorSteps = qCeil( ( max - min ) / stepSize );
     }
 
-    const QwtDate::IntervalType intvType = 
+    const QwtDate::IntervalType intvType =
         intervalType( from, to, maxMajorSteps );
 
     QwtScaleDiv scaleDiv;
@@ -1030,7 +1030,7 @@ QwtScaleDiv QwtDateScaleEngine::divideScale( double x1, double x2,
         const QDateTime minDate = QwtDate::floor( from, intvType );
         const QDateTime maxDate = QwtDate::ceil( to, intvType );
 
-        scaleDiv = buildScaleDiv( minDate, maxDate, 
+        scaleDiv = buildScaleDiv( minDate, maxDate,
             maxMajorSteps, maxMinorSteps, intvType );
 
         // scaleDiv has been calculated from an extended interval
@@ -1045,21 +1045,21 @@ QwtScaleDiv QwtDateScaleEngine::divideScale( double x1, double x2,
     return scaleDiv;
 }
 
-QwtScaleDiv QwtDateScaleEngine::buildScaleDiv( 
+QwtScaleDiv QwtDateScaleEngine::buildScaleDiv(
     const QDateTime &minDate, const QDateTime &maxDate,
     int maxMajorSteps, int maxMinorSteps,
     QwtDate::IntervalType intervalType ) const
 {
     // calculate the step size
-    const double stepSize = qwtDivideScale( 
-        qwtIntervalWidth( minDate, maxDate, intervalType ), 
+    const double stepSize = qwtDivideScale(
+        qwtIntervalWidth( minDate, maxDate, intervalType ),
         maxMajorSteps, intervalType );
 
     // align minDate to the step size
     QDateTime dt0 = alignDate( minDate, stepSize, intervalType, false );
     if ( !dt0.isValid() )
     {
-        // the floored date is out of the range of a 
+        // the floored date is out of the range of a
         // QDateTime - we ceil instead.
         dt0 = alignDate( minDate, stepSize, intervalType, true );
     }
@@ -1068,7 +1068,7 @@ QwtScaleDiv QwtDateScaleEngine::buildScaleDiv(
 
     if ( intervalType <= QwtDate::Week )
     {
-        scaleDiv = qwtDivideToSeconds( dt0, maxDate, 
+        scaleDiv = qwtDivideToSeconds( dt0, maxDate,
             stepSize, maxMinorSteps, intervalType );
     }
     else
@@ -1093,7 +1093,7 @@ QwtScaleDiv QwtDateScaleEngine::buildScaleDiv(
   Align a date/time value for a step size
 
   For Qt::Day alignments there is no "natural day 0" -
-  instead the first day of the year is used to avoid jumping 
+  instead the first day of the year is used to avoid jumping
   major ticks positions when panning a scale. For other alignments
   ( f.e according to the first day of the month ) alignDate()
   has to be overloaded.
@@ -1105,8 +1105,8 @@ QwtScaleDiv QwtDateScaleEngine::buildScaleDiv(
 
   \return Aligned date/time value
  */
-QDateTime QwtDateScaleEngine::alignDate( 
-    const QDateTime &dateTime, double stepSize, 
+QDateTime QwtDateScaleEngine::alignDate(
+    const QDateTime &dateTime, double stepSize,
     QwtDate::IntervalType intervalType, bool up ) const
 {
     // what about: (year == 1582 && month == 10 && day > 4 && day < 15) ??
@@ -1122,7 +1122,7 @@ QDateTime QwtDateScaleEngine::alignDate(
     {
         case QwtDate::Millisecond:
         {
-            const int ms = qwtAlignValue( 
+            const int ms = qwtAlignValue(
                 dt.time().msec(), stepSize, up ) ;
 
             dt = QwtDate::floor( dateTime, QwtDate::Second );
@@ -1292,7 +1292,7 @@ QDateTime QwtDateScaleEngine::toDateTime( double value ) const
     QDateTime dt = QwtDate::toDateTime( value, d_data->timeSpec );
     if ( !dt.isValid() )
     {
-        const QDate date = ( value <= 0.0 ) 
+        const QDate date = ( value <= 0.0 )
             ? QwtDate::minDate() : QwtDate::maxDate();
 
         dt = QDateTime( date, QTime( 0, 0 ), d_data->timeSpec );
