@@ -26,6 +26,8 @@ public:
     int maxMajor;
     int maxMinor;
 
+    bool isValid;
+
     QwtScaleDiv scaleDiv;
     QwtScaleEngine *scaleEngine;
     QwtScaleWidget *scaleWidget;
@@ -53,13 +55,21 @@ void QwtPlot::initAxesData()
     d_axisData[xTop]->scaleWidget->setObjectName( "QwtPlotAxisXTop" );
     d_axisData[xBottom]->scaleWidget->setObjectName( "QwtPlotAxisXBottom" );
 
+#if 1
+    // better find the font sizes from the application font
     QFont fscl( fontInfo().family(), 10 );
     QFont fttl( fontInfo().family(), 12, QFont::Bold );
+#endif
 
     for ( axisId = 0; axisId < axisCnt; axisId++ )
     {
         AxisData &d = *d_axisData[axisId];
 
+        d.scaleEngine = new QwtLinearScaleEngine;
+
+        d.scaleWidget->setTransformation(
+            d.scaleEngine->transformation() );
+
         d.scaleWidget->setFont( fscl );
         d.scaleWidget->setMargin( 2 );
 
@@ -76,9 +86,8 @@ void QwtPlot::initAxesData()
         d.maxMinor = 5;
         d.maxMajor = 8;
 
-        d.scaleEngine = new QwtLinearScaleEngine;
 
-        d.scaleDiv.invalidate();
+        d.isValid = false;
     }
 
     d_axisData[yLeft]->isEnabled = true;
@@ -98,8 +107,8 @@ void QwtPlot::deleteAxesData()
 }
 
 /*!
-  \return specified axis, or NULL if axisId is invalid.
-  \param axisId axis index
+  \return Scale widget of the specified axis, or NULL if axisId is invalid.
+  \param axisId Axis index
 */
 const QwtScaleWidget *QwtPlot::axisWidget( int axisId ) const
 {
@@ -110,8 +119,8 @@ const QwtScaleWidget *QwtPlot::axisWidget( int axisId ) const
 }
 
 /*!
-  \return specified axis, or NULL if axisId is invalid.
-  \param axisId axis index
+  \return Scale widget of the specified axis, or NULL if axisId is invalid.
+  \param axisId Axis index
 */
 QwtScaleWidget *QwtPlot::axisWidget( int axisId )
 {
@@ -122,9 +131,9 @@ QwtScaleWidget *QwtPlot::axisWidget( int axisId )
 }
 
 /*!
-   Change the scale engine for an axis
+  Change the scale engine for an axis
 
-  \param axisId axis index
+  \param axisId Axis index
   \param scaleEngine Scale engine
 
   \sa axisScaleEngine()
@@ -138,14 +147,17 @@ void QwtPlot::setAxisScaleEngine( int axisId, QwtScaleEngine *scaleEngine )
         delete d.scaleEngine;
         d.scaleEngine = scaleEngine;
 
-        d.scaleDiv.invalidate();
+        d_axisData[axisId]->scaleWidget->setTransformation(
+            scaleEngine->transformation() );
+
+        d.isValid = false;
 
         autoRefresh();
     }
 }
 
 /*!
-  \param axisId axis index
+  \param axisId Axis index
   \return Scale engine for a specific axis
 */
 QwtScaleEngine *QwtPlot::axisScaleEngine( int axisId )
@@ -157,7 +169,7 @@ QwtScaleEngine *QwtPlot::axisScaleEngine( int axisId )
 }
 
 /*!
-  \param axisId axis index
+  \param axisId Axis index
   \return Scale engine for a specific axis
 */
 const QwtScaleEngine *QwtPlot::axisScaleEngine( int axisId ) const
@@ -168,8 +180,8 @@ const QwtScaleEngine *QwtPlot::axisScaleEngine( int axisId ) const
         return NULL;
 }
 /*!
-  \return \c true if autoscaling is enabled
-  \param axisId axis index
+  \return \c True, if autoscaling is enabled
+  \param axisId Axis index
 */
 bool QwtPlot::axisAutoScale( int axisId ) const
 {
@@ -177,12 +189,11 @@ bool QwtPlot::axisAutoScale( int axisId ) const
         return d_axisData[axisId]->doAutoScale;
     else
         return false;
-
 }
 
 /*!
-  \return \c true if a specified axis is enabled
-  \param axisId axis index
+  \return \c True, if a specified axis is enabled
+  \param axisId Axis index
 */
 bool QwtPlot::axisEnabled( int axisId ) const
 {
@@ -193,8 +204,8 @@ bool QwtPlot::axisEnabled( int axisId ) const
 }
 
 /*!
-  \return the font of the scale labels for a specified axis
-  \param axisId axis index
+  \return The font of the scale labels for a specified axis
+  \param axisId Axis index
 */
 QFont QwtPlot::axisFont( int axisId ) const
 {
@@ -206,9 +217,9 @@ QFont QwtPlot::axisFont( int axisId ) const
 }
 
 /*!
-  \return the maximum number of major ticks for a specified axis
-  \param axisId axis index
-  \sa setAxisMaxMajor()
+  \return The maximum number of major ticks for a specified axis
+  \param axisId Axis index
+  \sa setAxisMaxMajor(), QwtScaleEngine::divideScale()
 */
 int QwtPlot::axisMaxMajor( int axisId ) const
 {
@@ -220,8 +231,8 @@ int QwtPlot::axisMaxMajor( int axisId ) const
 
 /*!
   \return the maximum number of minor ticks for a specified axis
-  \param axisId axis index
-  \sa setAxisMaxMinor()
+  \param axisId Axis index
+  \sa setAxisMaxMinor(), QwtScaleEngine::divideScale()
 */
 int QwtPlot::axisMaxMinor( int axisId ) const
 {
@@ -234,46 +245,24 @@ int QwtPlot::axisMaxMinor( int axisId ) const
 /*!
   \brief Return the scale division of a specified axis
 
-  axisScaleDiv(axisId)->lowerBound(), axisScaleDiv(axisId)->upperBound()
+  axisScaleDiv(axisId).lowerBound(), axisScaleDiv(axisId).upperBound()
   are the current limits of the axis scale.
 
-  \param axisId axis index
+  \param axisId Axis index
   \return Scale division
 
-  \sa QwtScaleDiv, setAxisScaleDiv()
+  \sa QwtScaleDiv, setAxisScaleDiv(), QwtScaleEngine::divideScale()
 */
-const QwtScaleDiv *QwtPlot::axisScaleDiv( int axisId ) const
+const QwtScaleDiv &QwtPlot::axisScaleDiv( int axisId ) const
 {
-    if ( !axisValid( axisId ) )
-        return NULL;
-
-    return &d_axisData[axisId]->scaleDiv;
+    return d_axisData[axisId]->scaleDiv;
 }
 
 /*!
-  \brief Return the scale division of a specified axis
+  \brief Return the scale draw of a specified axis
 
-  axisScaleDiv(axisId)->lowerBound(), axisScaleDiv(axisId)->upperBound()
-  are the current limits of the axis scale.
-
-  \param axisId axis index
-  \return Scale division
-
-  \sa QwtScaleDiv, setAxisScaleDiv()
-*/
-QwtScaleDiv *QwtPlot::axisScaleDiv( int axisId )
-{
-    if ( !axisValid( axisId ) )
-        return NULL;
-
-    return &d_axisData[axisId]->scaleDiv;
-}
-
-/*!
-  \returns the scale draw of a specified axis
-  \param axisId axis index
-  \return specified scaleDraw for axis, or NULL if axis is invalid.
-  \sa QwtScaleDraw
+  \param axisId Axis index
+  \return Specified scaleDraw for axis, or NULL if axis is invalid.
 */
 const QwtScaleDraw *QwtPlot::axisScaleDraw( int axisId ) const
 {
@@ -284,10 +273,10 @@ const QwtScaleDraw *QwtPlot::axisScaleDraw( int axisId ) const
 }
 
 /*!
-  \returns the scale draw of a specified axis
-  \param axisId axis index
-  \return specified scaleDraw for axis, or NULL if axis is invalid.
-  \sa QwtScaleDraw
+  \brief Return the scale draw of a specified axis
+
+  \param axisId Axis index
+  \return Specified scaleDraw for axis, or NULL if axis is invalid.
 */
 QwtScaleDraw *QwtPlot::axisScaleDraw( int axisId )
 {
@@ -298,14 +287,14 @@ QwtScaleDraw *QwtPlot::axisScaleDraw( int axisId )
 }
 
 /*!
-   Return the step size parameter, that has been set
-   in setAxisScale. This doesn't need to be the step size
-   of the current scale.
+  \brief Return the step size parameter that has been set in setAxisScale.
 
-  \param axisId axis index
+  This doesn't need to be the step size of the current scale.
+
+  \param axisId Axis index
   \return step size parameter value
 
-   \sa setAxisScale()
+   \sa setAxisScale(), QwtScaleEngine::divideScale()
 */
 double QwtPlot::axisStepSize( int axisId ) const
 {
@@ -319,8 +308,8 @@ double QwtPlot::axisStepSize( int axisId ) const
   \brief Return the current interval of the specified axis
 
   This is only a convenience function for axisScaleDiv( axisId )->interval();
-  
-  \param axisId axis index
+
+  \param axisId Axis index
   \return Scale interval
 
   \sa QwtScaleDiv, axisScaleDiv()
@@ -334,8 +323,8 @@ QwtInterval QwtPlot::axisInterval( int axisId ) const
 }
 
 /*!
-  \return the title of a specified axis
-  \param axisId axis index
+  \return Title of a specified axis
+  \param axisId Axis index
 */
 QwtText QwtPlot::axisTitle( int axisId ) const
 {
@@ -354,7 +343,8 @@ QwtText QwtPlot::axisTitle( int axisId ) const
   into values works as normal.
 
   Only xBottom and yLeft are enabled by default.
-  \param axisId axis index
+
+  \param axisId Axis index
   \param tf \c true (enabled) or \c false (disabled)
 */
 void QwtPlot::enableAxis( int axisId, bool tf )
@@ -369,8 +359,12 @@ void QwtPlot::enableAxis( int axisId, bool tf )
 /*!
   Transform the x or y coordinate of a position in the
   drawing region into a value.
-  \param axisId axis index
+
+  \param axisId Axis index
   \param pos position
+
+  \return Position as axis coordinate
+
   \warning The position can be an x or a y coordinate,
            depending on the specified axis.
 */
@@ -385,9 +379,10 @@ double QwtPlot::invTransform( int axisId, int pos ) const
 
 /*!
   \brief Transform a value into a coordinate in the plotting region
-  \param axisId axis index
+
+  \param axisId Axis index
   \param value value
-  \return X or y coordinate in the plotting region corresponding
+  \return X or Y coordinate in the plotting region corresponding
           to the value.
 */
 double QwtPlot::transform( int axisId, double value ) const
@@ -400,15 +395,16 @@ double QwtPlot::transform( int axisId, double value ) const
 
 /*!
   \brief Change the font of an axis
-  \param axisId axis index
-  \param f font
+
+  \param axisId Axis index
+  \param font Font
   \warning This function changes the font of the tick labels,
            not of the axis title.
 */
-void QwtPlot::setAxisFont( int axisId, const QFont &f )
+void QwtPlot::setAxisFont( int axisId, const QFont &font )
 {
     if ( axisValid( axisId ) )
-        axisWidget( axisId )->setFont( f );
+        axisWidget( axisId )->setFont( font );
 }
 
 /*!
@@ -417,7 +413,7 @@ void QwtPlot::setAxisFont( int axisId, const QFont &f )
   This member function is used to switch back to autoscaling mode
   after a fixed scale has been set. Autoscaling is enabled by default.
 
-  \param axisId axis index
+  \param axisId Axis index
   \param on On/Off
   \sa setAxisScale(), setAxisScaleDiv(), updateAxes()
 
@@ -435,12 +431,18 @@ void QwtPlot::setAxisAutoScale( int axisId, bool on )
 
 /*!
   \brief Disable autoscaling and specify a fixed scale for a selected axis.
-  \param axisId axis index
-  \param min
-  \param max minimum and maximum of the scale
+
+  In updateAxes() the scale engine calculates a scale division from the
+  specified parameters, that will be assigned to the scale widget. So
+  updates of the scale widget usually happen delayed with the next replot.
+
+  \param axisId Axis index
+  \param min Minimum of the scale
+  \param max Maximum of the scale
   \param stepSize Major step size. If <code>step == 0</code>, the step size is
-            calculated automatically using the maxMajor setting.
-  \sa setAxisMaxMajor(), setAxisAutoScale(), axisStepSize()
+                  calculated automatically using the maxMajor setting.
+
+  \sa setAxisMaxMajor(), setAxisAutoScale(), axisStepSize(), QwtScaleEngine::divideScale()
 */
 void QwtPlot::setAxisScale( int axisId, double min, double max, double stepSize )
 {
@@ -449,7 +451,7 @@ void QwtPlot::setAxisScale( int axisId, double min, double max, double stepSize
         AxisData &d = *d_axisData[axisId];
 
         d.doAutoScale = false;
-        d.scaleDiv.invalidate();
+        d.isValid = false;
 
         d.minValue = min;
         d.maxValue = max;
@@ -461,8 +463,14 @@ void QwtPlot::setAxisScale( int axisId, double min, double max, double stepSize
 
 /*!
   \brief Disable autoscaling and specify a fixed scale for a selected axis.
-  \param axisId axis index
+
+  The scale division will be stored locally only until the next call
+  of updateAxes(). So updates of the scale widget usually happen delayed with
+  the next replot.
+
+  \param axisId Axis index
   \param scaleDiv Scale division
+
   \sa setAxisScale(), setAxisAutoScale()
 */
 void QwtPlot::setAxisScaleDiv( int axisId, const QwtScaleDiv &scaleDiv )
@@ -473,6 +481,7 @@ void QwtPlot::setAxisScaleDiv( int axisId, const QwtScaleDiv &scaleDiv )
 
         d.doAutoScale = false;
         d.scaleDiv = scaleDiv;
+        d.isValid = true;
 
         autoRefresh();
     }
@@ -480,8 +489,9 @@ void QwtPlot::setAxisScaleDiv( int axisId, const QwtScaleDiv &scaleDiv )
 
 /*!
   \brief Set a scale draw
-  \param axisId axis index
-  \param scaleDraw object responsible for drawing scales.
+
+  \param axisId Axis index
+  \param scaleDraw Object responsible for drawing scales.
 
   By passing scaleDraw it is possible to extend QwtScaleDraw
   functionality and let it take place in QwtPlot. Please note
@@ -504,8 +514,10 @@ void QwtPlot::setAxisScaleDraw( int axisId, QwtScaleDraw *scaleDraw )
 
 /*!
   Change the alignment of the tick labels
-  \param axisId axis index
+
+  \param axisId Axis index
   \param alignment Or'd Qt::AlignmentFlags see <qnamespace.h>
+
   \sa QwtScaleDraw::setLabelAlignment()
 */
 void QwtPlot::setAxisLabelAlignment( int axisId, Qt::Alignment alignment )
@@ -516,9 +528,11 @@ void QwtPlot::setAxisLabelAlignment( int axisId, Qt::Alignment alignment )
 
 /*!
   Rotate all tick labels
-  \param axisId axis index
+
+  \param axisId Axis index
   \param rotation Angle in degrees. When changing the label rotation,
                   the label alignment might be adjusted too.
+
   \sa QwtScaleDraw::setLabelRotation(), setAxisLabelAlignment()
 */
 void QwtPlot::setAxisLabelRotation( int axisId, double rotation )
@@ -530,8 +544,9 @@ void QwtPlot::setAxisLabelRotation( int axisId, double rotation )
 /*!
   Set the maximum number of minor scale intervals for a specified axis
 
-  \param axisId axis index
-  \param maxMinor maximum number of minor steps
+  \param axisId Axis index
+  \param maxMinor Maximum number of minor steps
+
   \sa axisMaxMinor()
 */
 void QwtPlot::setAxisMaxMinor( int axisId, int maxMinor )
@@ -544,7 +559,7 @@ void QwtPlot::setAxisMaxMinor( int axisId, int maxMinor )
         if ( maxMinor != d.maxMinor )
         {
             d.maxMinor = maxMinor;
-            d.scaleDiv.invalidate();
+            d.isValid = false;
             autoRefresh();
         }
     }
@@ -553,8 +568,9 @@ void QwtPlot::setAxisMaxMinor( int axisId, int maxMinor )
 /*!
   Set the maximum number of major scale intervals for a specified axis
 
-  \param axisId axis index
-  \param maxMajor maximum number of major steps
+  \param axisId Axis index
+  \param maxMajor Maximum number of major steps
+
   \sa axisMaxMajor()
 */
 void QwtPlot::setAxisMaxMajor( int axisId, int maxMajor )
@@ -567,7 +583,7 @@ void QwtPlot::setAxisMaxMajor( int axisId, int maxMajor )
         if ( maxMajor != d.maxMajor )
         {
             d.maxMajor = maxMajor;
-            d.scaleDiv.invalidate();
+            d.isValid = false;
             autoRefresh();
         }
     }
@@ -575,7 +591,8 @@ void QwtPlot::setAxisMaxMajor( int axisId, int maxMajor )
 
 /*!
   \brief Change the title of a specified axis
-  \param axisId axis index
+
+  \param axisId Axis index
   \param title axis title
 */
 void QwtPlot::setAxisTitle( int axisId, const QString &title )
@@ -586,8 +603,9 @@ void QwtPlot::setAxisTitle( int axisId, const QString &title )
 
 /*!
   \brief Change the title of a specified axis
-  \param axisId axis index
-  \param title axis title
+
+  \param axisId Axis index
+  \param title Axis title
 */
 void QwtPlot::setAxisTitle( int axisId, const QwtText &title )
 {
@@ -595,7 +613,30 @@ void QwtPlot::setAxisTitle( int axisId, const QwtText &title )
         axisWidget( axisId )->setTitle( title );
 }
 
-//! Rebuild the scales
+/*!
+  \brief Rebuild the axes scales
+
+  In case of autoscaling the boundaries of a scale are calculated
+  from the bounding rectangles of all plot items, having the
+  QwtPlotItem::AutoScale flag enabled ( QwtScaleEngine::autoScale() ).
+  Then a scale division is calculated ( QwtScaleEngine::didvideScale() )
+  and assigned to scale widget.
+
+  When the scale boundaries have been assigned with setAxisScale() a
+  scale division is calculated ( QwtScaleEngine::didvideScale() )
+  for this interval and assigned to the scale widget.
+
+  When the scale has been set explicitly by setAxisScaleDiv() the
+  locally stored scale division gets assigned to the scale widget.
+
+  The scale widget indicates modifications by emitting a
+  QwtScaleWidget::scaleDivChanged() signal.
+
+  updateAxes() is usually called by replot().
+
+  \sa setAxisAutoScale(), setAxisScale(), setAxisScaleDiv(), replot()
+      QwtPlotItem::boundingRect()
+ */
 void QwtPlot::updateAxes()
 {
     // Find bounding interval of the item data
@@ -640,7 +681,7 @@ void QwtPlot::updateAxes()
 
         if ( d.doAutoScale && intv[axisId].isValid() )
         {
-            d.scaleDiv.invalidate();
+            d.isValid = false;
 
             minValue = intv[axisId].minValue();
             maxValue = intv[axisId].maxValue();
@@ -648,16 +689,16 @@ void QwtPlot::updateAxes()
             d.scaleEngine->autoScale( d.maxMajor,
                 minValue, maxValue, stepSize );
         }
-        if ( !d.scaleDiv.isValid() )
+        if ( !d.isValid )
         {
             d.scaleDiv = d.scaleEngine->divideScale(
                 minValue, maxValue,
                 d.maxMajor, d.maxMinor, stepSize );
+            d.isValid = true;
         }
 
         QwtScaleWidget *scaleWidget = axisWidget( axisId );
-        scaleWidget->setScaleDiv(
-            d.scaleEngine->transformation(), d.scaleDiv );
+        scaleWidget->setScaleDiv( d.scaleDiv );
 
         int startDist, endDist;
         scaleWidget->getBorderDistHint( startDist, endDist );
@@ -667,8 +708,11 @@ void QwtPlot::updateAxes()
     for ( it = itmList.begin(); it != itmList.end(); ++it )
     {
         QwtPlotItem *item = *it;
-        item->updateScaleDiv( *axisScaleDiv( item->xAxis() ),
-            *axisScaleDiv( item->yAxis() ) );
+        if ( item->testItemInterest( QwtPlotItem::ScaleInterest ) )
+        {
+            item->updateScaleDiv( axisScaleDiv( item->xAxis() ),
+                axisScaleDiv( item->yAxis() ) );
+        }
     }
 }
 
