@@ -10,9 +10,9 @@
 #include "qwt_plot_item.h"
 #include "qwt_text.h"
 #include "qwt_plot.h"
-#include "qwt_legend.h"
-#include "qwt_legend_item.h"
+#include "qwt_legend_data.h"
 #include "qwt_scale_div.h"
+#include "qwt_graphic.h"
 #include <qpainter.h>
 
 class QwtPlotItem::PrivateData
@@ -22,24 +22,33 @@ public:
         plot( NULL ),
         isVisible( true ),
         attributes( 0 ),
+        interests( 0 ),
         renderHints( 0 ),
+        renderThreadCount( 1 ),
         z( 0.0 ),
         xAxis( QwtPlot::xBottom ),
-        yAxis( QwtPlot::yLeft )
+        yAxis( QwtPlot::yLeft ),
+        legendIconSize( 8, 8 )
     {
     }
 
     mutable QwtPlot *plot;
 
     bool isVisible;
+
     QwtPlotItem::ItemAttributes attributes;
+    QwtPlotItem::ItemInterests interests;
+
     QwtPlotItem::RenderHints renderHints;
+    uint renderThreadCount;
+
     double z;
 
     int xAxis;
     int yAxis;
 
     QwtText title;
+    QSize legendIconSize;
 };
 
 /*!
@@ -75,32 +84,17 @@ void QwtPlotItem::attach( QwtPlot *plot )
     if ( plot == d_data->plot )
         return;
 
-    // remove the item from the previous plot
-
     if ( d_data->plot )
-    {
-        if ( d_data->plot->legend() )
-            d_data->plot->legend()->remove( this );
-
         d_data->plot->attachItem( this, false );
 
-        if ( d_data->plot->autoReplot() )
-            d_data->plot->update();
-    }
-
     d_data->plot = plot;
 
     if ( d_data->plot )
-    {
-        // insert the item into the current plot
-
         d_data->plot->attachItem( this, true );
-        itemChanged();
-    }
 }
 
 /*!
-   \brief This method detaches a QwtPlotItem from any 
+   \brief This method detaches a QwtPlotItem from any
           QwtPlot it has been associated with.
 
    detach() is equivalent to calling attach( NULL )
@@ -190,7 +184,11 @@ void QwtPlotItem::setTitle( const QwtText &title )
     if ( d_data->title != title )
     {
         d_data->title = title;
+
+        legendChanged();
+#if 0
         itemChanged();
+#endif
     }
 }
 
@@ -209,17 +207,20 @@ const QwtText &QwtPlotItem::title() const
    \param attribute Attribute type
    \param on true/false
 
-   \sa testItemAttribute(), ItemAttribute
+   \sa testItemAttribute(), ItemInterest
 */
 void QwtPlotItem::setItemAttribute( ItemAttribute attribute, bool on )
 {
-    if ( bool( d_data->attributes & attribute ) != on )
+    if ( d_data->attributes.testFlag( attribute ) != on )
     {
         if ( on )
             d_data->attributes |= attribute;
         else
             d_data->attributes &= ~attribute;
 
+        if ( attribute == QwtPlotItem::Legend )
+            legendChanged();
+
         itemChanged();
     }
 }
@@ -229,11 +230,44 @@ void QwtPlotItem::setItemAttribute( ItemAttribute attribute, bool on )
 
    \param attribute Attribute type
    \return true/false
-   \sa setItemAttribute(), ItemAttribute
+   \sa setItemAttribute(), ItemInterest
 */
 bool QwtPlotItem::testItemAttribute( ItemAttribute attribute ) const
 {
-    return ( d_data->attributes & attribute );
+    return d_data->attributes.testFlag( attribute );
+}
+
+/*!
+   Toggle an item interest
+
+   \param interest Interest type
+   \param on true/false
+
+   \sa testItemInterest(), ItemAttribute
+*/
+void QwtPlotItem::setItemInterest( ItemInterest interest, bool on )
+{
+    if ( d_data->interests.testFlag( interest ) != on )
+    {
+        if ( on )
+            d_data->interests |= interest;
+        else
+            d_data->interests &= ~interest;
+
+        itemChanged();
+    }
+}
+
+/*!
+   Test an item interest
+
+   \param interest Interest type
+   \return true/false
+   \sa setItemInterest(), ItemAttribute
+*/
+bool QwtPlotItem::testItemInterest( ItemInterest interest ) const
+{
+    return d_data->interests.testFlag( interest );
 }
 
 /*!
@@ -246,7 +280,7 @@ bool QwtPlotItem::testItemAttribute( ItemAttribute attribute ) const
 */
 void QwtPlotItem::setRenderHint( RenderHint hint, bool on )
 {
-    if ( ( ( d_data->renderHints & hint ) != 0 ) != on )
+    if ( d_data->renderHints.testFlag( hint ) != on )
     {
         if ( on )
             d_data->renderHints |= hint;
@@ -266,7 +300,109 @@ void QwtPlotItem::setRenderHint( RenderHint hint, bool on )
 */
 bool QwtPlotItem::testRenderHint( RenderHint hint ) const
 {
-    return ( d_data->renderHints & hint );
+    return d_data->renderHints.testFlag( hint );
+}
+
+/*!
+   On multi core systems rendering of certain plot item
+   ( f.e QwtPlotRasterItem ) can be done in parallel in
+   several threads.
+
+   The default setting is set to 1.
+
+   \param numThreads Number of threads to be used for rendering.
+                     If numThreads is set to 0, the system specific
+                     ideal thread count is used.
+
+   The default thread count is 1 ( = no additional threads )
+*/
+void QwtPlotItem::setRenderThreadCount( uint numThreads )
+{
+    d_data->renderThreadCount = numThreads;
+}
+
+/*!
+   \return Number of threads to be used for rendering.
+           If numThreads() is set to 0, the system specific
+           ideal thread count is used.
+*/
+uint QwtPlotItem::renderThreadCount() const
+{
+    return d_data->renderThreadCount;
+}
+
+/*!
+   Set the size of the legend icon
+
+   The default setting is 8x8 pixels
+
+   \param size Size
+   \sa legendIconSize(), legendIcon()
+*/
+void QwtPlotItem::setLegendIconSize( const QSize &size )
+{
+    if ( d_data->legendIconSize != size )
+    {
+        d_data->legendIconSize = size;
+        legendChanged();
+    }
+}
+
+/*!
+   \return Legend icon size
+   \sa setLegendIconSize(), legendIcon()
+*/
+QSize QwtPlotItem::legendIconSize() const
+{
+    return d_data->legendIconSize;
+}
+
+/*!
+   \return Icon representing the item on the legend
+
+   The default implementation returns an invalid icon
+
+   \param index Index of the legend entry
+                ( usually there is only one )
+   \param size Icon size
+
+   \sa setLegendIconSize(), legendData()
+ */
+QwtGraphic QwtPlotItem::legendIcon(
+    int index, const QSizeF &size ) const
+{
+    Q_UNUSED( index )
+    Q_UNUSED( size )
+
+    return QwtGraphic();
+}
+
+/*!
+   \brief Return a default icon from a brush
+
+   The default icon is a filled rectangle used
+   in several derived classes as legendIcon().
+
+   \param brush Fill brush
+   \param size Icon size
+
+   \return A filled rectangle
+ */
+QwtGraphic QwtPlotItem::defaultIcon(
+    const QBrush &brush, const QSizeF &size ) const
+{
+    QwtGraphic icon;
+    if ( !size.isEmpty() )
+    {
+        icon.setDefaultSize( size );
+
+        QRectF r( 0, 0, size.width(), size.height() );
+
+        QPainter painter( &icon );
+        painter.fillRect( r, brush );
+    }
+
+    return icon;
 }
 
 //! Show the item
@@ -306,26 +442,31 @@ bool QwtPlotItem::isVisible() const
 }
 
 /*!
-   Update the legend and call QwtPlot::autoRefresh for the
+   Update the legend and call QwtPlot::autoRefresh() for the
    parent plot.
 
-   \sa updateLegend()
+   \sa QwtPlot::legendChanged(), QwtPlot::autoRefresh()
 */
 void QwtPlotItem::itemChanged()
 {
     if ( d_data->plot )
-    {
-        if ( d_data->plot->legend() )
-            updateLegend( d_data->plot->legend() );
-
         d_data->plot->autoRefresh();
-    }
+}
+
+/*!
+   Update the legend of the parent plot.
+   \sa QwtPlot::updateLegend(), itemChanged()
+*/
+void QwtPlotItem::legendChanged()
+{
+    if ( testItemAttribute( QwtPlotItem::Legend ) && d_data->plot )
+        d_data->plot->updateLegend( this );
 }
 
 /*!
    Set X and Y axis
 
-   The item will painted according to the coordinates its Axes.
+   The item will painted according to the coordinates of its Axes.
 
    \param xAxis X Axis ( QwtPlot::xBottom or QwtPlot::xTop )
    \param yAxis Y Axis ( QwtPlot::yLeft or QwtPlot::yRight )
@@ -399,95 +540,81 @@ QRectF QwtPlotItem::boundingRect() const
 }
 
 /*!
-   \brief Allocate the widget that represents the item on the legend
+   \brief Calculate a hint for the canvas margin
 
-   The default implementation returns a QwtLegendItem(), but an item 
-   could be represented by any type of widget,
-   by overloading legendItem() and updateLegend().
+   When the QwtPlotItem::Margins flag is enabled the plot item
+   indicates, that it needs some margins at the borders of the canvas.
+   This is f.e. used by bar charts to reserve space for displaying
+   the bars.
 
-   \return QwtLegendItem()
-   \sa updateLegend() QwtLegend()
-*/
-QWidget *QwtPlotItem::legendItem() const
+   The margins are in target device coordinates ( pixels on screen )
+
+   \param xMap Maps x-values into pixel coordinates.
+   \param yMap Maps y-values into pixel coordinates.
+   \param canvasRect Contents rectangle of the canvas in painter coordinates
+   \param left Returns the left margin
+   \param top Returns the top margin
+   \param right Returns the right margin
+   \param bottom Returns the bottom margin
+
+   The default implementation returns 0 for all margins
+
+   \sa QwtPlot::getCanvasMarginsHint(), QwtPlot::updateCanvasMargins()
+ */
+void QwtPlotItem::getCanvasMarginHint( const QwtScaleMap &xMap,
+    const QwtScaleMap &yMap, const QRectF &canvasRect,
+    double &left, double &top, double &right, double &bottom ) const
 {
-    QwtLegendItem *item = new QwtLegendItem;
-    if ( d_data->plot )
-    {
-        QObject::connect( item, SIGNAL( clicked() ),
-            d_data->plot, SLOT( legendItemClicked() ) );
-        QObject::connect( item, SIGNAL( checked( bool ) ),
-            d_data->plot, SLOT( legendItemChecked( bool ) ) );
-    }
-    return item;
+    Q_UNUSED( xMap );
+    Q_UNUSED( yMap );
+    Q_UNUSED( canvasRect );
+
+    // use QMargins, when we don't need to support Qt < 4.6 anymore
+    left = top = right = bottom = 0.0;
 }
 
 /*!
-   \brief Update the widget that represents the item on the legend
+   \brief Return all information, that is needed to represent
+          the item on the legend
 
-   updateLegend() is called from itemChanged() to adopt the widget
-   representing the item on the legend to its new configuration.
+   Most items are represented by one entry on the legend
+   showing an icon and a text, but f.e. QwtPlotMultiBarChart
+   displays one entry for each bar.
 
-   The default implementation updates a QwtLegendItem(), 
-   but an item could be represented by any type of widget,
-   by overloading legendItem() and updateLegend().
+   QwtLegendData is basically a list of QVariants that makes it
+   possible to overload and reimplement legendData() to
+   return almost any type of information, that is understood
+   by the receiver that acts as the legend.
 
-   \param legend Legend
+   The default implementation returns one entry with
+   the title() of the item and the legendIcon().
 
-   \sa legendItem(), itemChanged(), QwtLegend()
-*/
-void QwtPlotItem::updateLegend( QwtLegend *legend ) const
+   \return Data, that is needed to represent the item on the legend
+   \sa title(), legendIcon(), QwtLegend, QwtPlotLegendItem
+ */
+QList<QwtLegendData> QwtPlotItem::legendData() const
 {
-    if ( legend == NULL )
-        return;
+    QwtLegendData data;
 
-    QWidget *lgdItem = legend->find( this );
-    if ( testItemAttribute( QwtPlotItem::Legend ) )
+    QwtText label = title();
+    label.setRenderFlags( label.renderFlags() & Qt::AlignLeft );
+
+    QVariant titleValue;
+    qVariantSetValue( titleValue, label );
+    data.setValue( QwtLegendData::TitleRole, titleValue );
+
+    const QwtGraphic graphic = legendIcon( 0, legendIconSize() );
+    if ( !graphic.isNull() )
     {
-        if ( lgdItem == NULL )
-        {
-            lgdItem = legendItem();
-            if ( lgdItem )
-                legend->insert( this, lgdItem );
-        }
-
-        QwtLegendItem *label = qobject_cast<QwtLegendItem *>( lgdItem );
-        if ( label )
-        {
-            // paint the identifier
-            const QSize sz = label->identifierSize();
-
-            QPixmap identifier( sz.width(), sz.height() );
-            identifier.fill( Qt::transparent );
-
-            QPainter painter( &identifier );
-            painter.setRenderHint( QPainter::Antialiasing,
-                testRenderHint( QwtPlotItem::RenderAntialiased ) );
-            drawLegendIdentifier( &painter,
-                QRect( 0, 0, sz.width(), sz.height() ) );
-            painter.end();
-
-            const bool doUpdate = label->updatesEnabled();
-            if ( doUpdate )
-                label->setUpdatesEnabled( false );
-
-            label->setText( title() );
-            label->setIdentifier( identifier );
-            label->setItemMode( legend->itemMode() );
-
-            if ( doUpdate )
-                label->setUpdatesEnabled( true );
-
-            label->update();
-        }
-    }
-    else
-    {
-        if ( lgdItem )
-        {
-            lgdItem->hide();
-            lgdItem->deleteLater();
-        }
+        QVariant iconValue;
+        qVariantSetValue( iconValue, graphic );
+        data.setValue( QwtLegendData::IconRole, iconValue );
     }
+
+    QList<QwtLegendData> list;
+    list += data;
+
+    return list;
 }
 
 /*!
@@ -498,10 +625,13 @@ void QwtPlotItem::updateLegend( QwtLegend *legend ) const
    on the scale division (like QwtPlotGrid()) have to reimplement
    updateScaleDiv()
 
+   updateScaleDiv() is only called when the ScaleInterest interest
+   is enabled. The default implementation does nothing.
+
    \param xScaleDiv Scale division of the x-axis
    \param yScaleDiv Scale division of the y-axis
 
-   \sa QwtPlot::updateAxes()
+   \sa QwtPlot::updateAxes(), ScaleInterest
 */
 void QwtPlotItem::updateScaleDiv( const QwtScaleDiv &xScaleDiv,
     const QwtScaleDiv &yScaleDiv )
@@ -511,10 +641,35 @@ void QwtPlotItem::updateScaleDiv( const QwtScaleDiv &xScaleDiv,
 }
 
 /*!
-   \brief Calculate the bounding scale rect of 2 maps
+   \brief Update the item to changes of the legend info
 
-   \param xMap X map
-   \param yMap Y map
+   Plot items that want to display a legend ( not those, that want to
+   be displayed on a legend ! ) will have to implement updateLegend().
+
+   updateLegend() is only called when the LegendInterest interest
+   is enabled. The default implementation does nothing.
+
+   \param item Plot item to be displayed on a legend
+   \param data Attributes how to display item on the legend
+
+   \sa QwtPlotLegendItem
+
+   \note Plot items, that want to be displayed on a legend
+         need to enable the QwtPlotItem::Legend flag and to implement
+         legendData() and legendIcon()
+ */
+void QwtPlotItem::updateLegend( const QwtPlotItem *item,
+    const QList<QwtLegendData> &data )
+{
+    Q_UNUSED( item );
+    Q_UNUSED( data );
+}
+
+/*!
+   \brief Calculate the bounding scale rectangle of 2 maps
+
+   \param xMap Maps x-values into pixel coordinates.
+   \param yMap Maps y-values into pixel coordinates.
 
    \return Bounding scale rect of the scale maps, not normalized
 */
@@ -526,12 +681,12 @@ QRectF QwtPlotItem::scaleRect( const QwtScaleMap &xMap,
 }
 
 /*!
-   \brief Calculate the bounding paint rect of 2 maps
+   \brief Calculate the bounding paint rectangle of 2 maps
 
-   \param xMap X map
-   \param yMap Y map
+   \param xMap Maps x-values into pixel coordinates.
+   \param yMap Maps y-values into pixel coordinates.
 
-   \return Bounding paint rect of the scale maps, not normalized
+   \return Bounding paint rectangle of the scale maps, not normalized
 */
 QRectF QwtPlotItem::paintRect( const QwtScaleMap &xMap,
     const QwtScaleMap &yMap ) const
