@@ -13,8 +13,6 @@
 #include "qwt_symbol.h"
 #include "qwt_text.h"
 #include "qwt_math.h"
-#include "qwt_legend.h"
-#include "qwt_legend_item.h"
 #include <qpainter.h>
 
 class QwtPlotMarker::PrivateData
@@ -50,8 +48,16 @@ public:
 };
 
 //! Sets alignment to Qt::AlignCenter, and style to QwtPlotMarker::NoLine
-QwtPlotMarker::QwtPlotMarker():
-    QwtPlotItem( QwtText( "Marker" ) )
+QwtPlotMarker::QwtPlotMarker( const QString &title ):
+    QwtPlotItem( QwtText( title ) )
+{
+    d_data = new PrivateData;
+    setZ( 30.0 );
+}
+
+//! Sets alignment to Qt::AlignCenter, and style to QwtPlotMarker::NoLine
+QwtPlotMarker::QwtPlotMarker( const QwtText &title ):
+    QwtPlotItem( title )
 {
     d_data = new PrivateData;
     setZ( 30.0 );
@@ -122,13 +128,13 @@ void QwtPlotMarker::setYValue( double y )
   \param painter Painter
   \param xMap x Scale Map
   \param yMap y Scale Map
-  \param canvasRect Contents rect of the canvas in painter coordinates
+  \param canvasRect Contents rectangle of the canvas in painter coordinates
 */
 void QwtPlotMarker::draw( QPainter *painter,
     const QwtScaleMap &xMap, const QwtScaleMap &yMap,
     const QRectF &canvasRect ) const
 {
-    const QPointF pos( xMap.transform( d_data->xValue ), 
+    const QPointF pos( xMap.transform( d_data->xValue ),
         yMap.transform( d_data->yValue ) );
 
     // draw lines
@@ -139,7 +145,13 @@ void QwtPlotMarker::draw( QPainter *painter,
     if ( d_data->symbol &&
         ( d_data->symbol->style() != QwtSymbol::NoSymbol ) )
     {
-        d_data->symbol->drawSymbol( painter, pos );
+        const QSizeF sz = d_data->symbol->size();
+
+        const QRectF clipRect = canvasRect.adjusted(
+            -sz.width(), -sz.height(), sz.width(), sz.height() );
+
+        if ( clipRect.contains( pos ) )
+            d_data->symbol->drawSymbol( painter, pos );
     }
 
     drawLabel( painter, canvasRect, pos );
@@ -149,7 +161,7 @@ void QwtPlotMarker::draw( QPainter *painter,
   Draw the lines marker
 
   \param painter Painter
-  \param canvasRect Contents rect of the canvas in painter coordinates
+  \param canvasRect Contents rectangle of the canvas in painter coordinates
   \param pos Position of the marker, translated into widget coordinates
 
   \sa drawLabel(), QwtSymbol::drawSymbol()
@@ -189,7 +201,7 @@ void QwtPlotMarker::drawLines( QPainter *painter,
   Align and draw the text label of the marker
 
   \param painter Painter
-  \param canvasRect Contents rect of the canvas in painter coordinates
+  \param canvasRect Contents rectangle of the canvas in painter coordinates
   \param pos Position of the marker, translated into widget coordinates
 
   \sa drawLabel(), QwtSymbol::drawSymbol()
@@ -325,7 +337,7 @@ void QwtPlotMarker::drawLabel( QPainter *painter,
 
 /*!
   \brief Set the line style
-  \param style Line style. 
+  \param style Line style.
   \sa lineStyle()
 */
 void QwtPlotMarker::setLineStyle( LineStyle style )
@@ -333,6 +345,8 @@ void QwtPlotMarker::setLineStyle( LineStyle style )
     if ( style != d_data->style )
     {
         d_data->style = style;
+
+        legendChanged();
         itemChanged();
     }
 }
@@ -357,6 +371,11 @@ void QwtPlotMarker::setSymbol( const QwtSymbol *symbol )
     {
         delete d_data->symbol;
         d_data->symbol = symbol;
+
+        if ( symbol )
+            setLegendIconSize( symbol->boundingRect().size() );
+
+        legendChanged();
         itemChanged();
     }
 }
@@ -372,7 +391,7 @@ const QwtSymbol *QwtPlotMarker::symbol() const
 
 /*!
   \brief Set the label
-  \param label label text
+  \param label Label text
   \sa label()
 */
 void QwtPlotMarker::setLabel( const QwtText& label )
@@ -404,7 +423,7 @@ QwtText QwtPlotMarker::label() const
 
   In all other styles the alignment is relative to the marker's position.
 
-  \param align Alignment. 
+  \param align Alignment.
   \sa labelAlignment(), labelOrientation()
 */
 void QwtPlotMarker::setLabelAlignment( Qt::Alignment align )
@@ -483,6 +502,24 @@ int QwtPlotMarker::spacing() const
     return d_data->spacing;
 }
 
+/*!
+  Build and assign a line pen
+
+  In Qt5 the default pen width is 1.0 ( 0.0 in Qt4 ) what makes it
+  non cosmetic ( see QPen::isCosmetic() ). This method has been introduced
+  to hide this incompatibility.
+
+  \param color Pen color
+  \param width Pen width
+  \param style Pen style
+
+  \sa pen(), brush()
+ */
+void QwtPlotMarker::setLinePen( const QColor &color, qreal width, Qt::PenStyle style )
+{
+    setLinePen( QPen( color, width, style ) );
+}
+
 /*!
   Specify a pen for the line.
 
@@ -494,6 +531,8 @@ void QwtPlotMarker::setLinePen( const QPen &pen )
     if ( pen != d_data->pen )
     {
         d_data->pen = pen;
+
+        legendChanged();
         itemChanged();
     }
 }
@@ -509,95 +548,75 @@ const QPen &QwtPlotMarker::linePen() const
 
 QRectF QwtPlotMarker::boundingRect() const
 {
-    return QRectF( d_data->xValue, d_data->yValue, 0.0, 0.0 );
-}
+    // width/height of -1 does not affect the autoscale calculation
 
-/*!
-   \brief Update the widget that represents the item on the legend
-
-   \param legend Legend
-   \sa drawLegendIdentifier(), legendItem(), itemChanged(), QwtLegend()
-
-   \note In the default setting QwtPlotItem::Legend is disabled 
-*/
-void QwtPlotMarker::updateLegend( QwtLegend *legend ) const
-{
-    if ( legend && testItemAttribute( QwtPlotItem::Legend )
-        && d_data->symbol && d_data->symbol->style() != QwtSymbol::NoSymbol )
+    switch (d_data->style)
     {
-        QWidget *lgdItem = legend->find( this );
-        if ( lgdItem == NULL )
-        {
-            lgdItem = legendItem();
-            if ( lgdItem )
-                legend->insert( this, lgdItem );
-        }
-        if ( lgdItem && lgdItem->inherits( "QwtLegendItem" ) )
-        {
-            QwtLegendItem *l = ( QwtLegendItem * )lgdItem;
-            l->setIdentifierSize( d_data->symbol->boundingSize() );
-        }
-    }
+        case QwtPlotMarker::HLine:
+            return QRectF( d_data->xValue, d_data->yValue, -1.0, 0.0 );
 
-    QwtPlotItem::updateLegend( legend );
+        case QwtPlotMarker::VLine:
+            return QRectF( d_data->xValue, d_data->yValue, 0.0, -1.0 );
+
+        default :
+            return QRectF( d_data->xValue, d_data->yValue, 0.0, 0.0 );
+    }
 }
 
 /*!
-  \brief Draw the identifier representing the marker on the legend
+   \return Icon representing the marker on the legend
 
-  \param painter Painter
-  \param rect Bounding rectangle for the identifier
+   \param index Index of the legend entry
+                ( usually there is only one )
+   \param size Icon size
 
-  \sa updateLegend(), QwtPlotItem::Legend
+   \sa setLegendIconSize(), legendData()
 */
-void QwtPlotMarker::drawLegendIdentifier(
-    QPainter *painter, const QRectF &rect ) const
+QwtGraphic QwtPlotMarker::legendIcon( int index,
+    const QSizeF &size ) const
 {
-    if ( rect.isEmpty() )
-        return;
+    Q_UNUSED( index );
 
-    painter->save();
-    painter->setClipRect( rect, Qt::IntersectClip );
+    if ( size.isEmpty() )
+        return QwtGraphic();
+
+    QwtGraphic icon;
+    icon.setDefaultSize( size );
+    icon.setRenderHint( QwtGraphic::RenderPensUnscaled, true );
+
+    QPainter painter( &icon );
+    painter.setRenderHint( QPainter::Antialiasing,
+        testRenderHint( QwtPlotItem::RenderAntialiased ) );
 
     if ( d_data->style != QwtPlotMarker::NoLine )
     {
-        painter->setPen( d_data->pen );
+        painter.setPen( d_data->pen );
 
         if ( d_data->style == QwtPlotMarker::HLine ||
             d_data->style == QwtPlotMarker::Cross )
         {
-            QwtPainter::drawLine( painter, rect.left(), rect.center().y(),
-                rect.right(), rect.center().y() );
+            const double y = 0.5 * size.height();
+
+            QwtPainter::drawLine( &painter,
+                0.0, y, size.width(), y );
         }
 
         if ( d_data->style == QwtPlotMarker::VLine ||
             d_data->style == QwtPlotMarker::Cross )
         {
-            QwtPainter::drawLine( painter, rect.center().x(), rect.top(),
-                rect.center().x(), rect.bottom() );
+            const double x = 0.5 * size.width();
+
+            QwtPainter::drawLine( &painter,
+                x, 0.0, x, size.height() );
         }
     }
 
-    if ( d_data->symbol && d_data->symbol->style() != QwtSymbol::NoSymbol )
+    if ( d_data->symbol )
     {
-        QSize symbolSize = d_data->symbol->boundingSize();
-        symbolSize -= QSize( 2, 2 );
-
-        // scale the symbol size down if it doesn't fit into rect.
-
-        double xRatio = 1.0;
-        if ( rect.width() < symbolSize.width() )
-            xRatio = rect.width() / symbolSize.width();
-        double yRatio = 1.0;
-        if ( rect.height() < symbolSize.height() )
-            yRatio = rect.height() / symbolSize.height();
-
-        const double ratio = qMin( xRatio, yRatio );
-
-        painter->scale( ratio, ratio );
-        d_data->symbol->drawSymbol( painter, rect.center() / ratio );
+        const QRect r( 0.0, 0.0, size.width(), size.height() );
+        d_data->symbol->drawSymbol( &painter, r );
     }
 
-    painter->restore();
+    return icon;
 }
 
