@@ -8,14 +8,18 @@
  *****************************************************************************/
 
 #include "qwt_plot_rasteritem.h"
-#include "qwt_legend.h"
-#include "qwt_legend_item.h"
 #include "qwt_scale_map.h"
 #include "qwt_painter.h"
 #include <qapplication.h>
 #include <qdesktopwidget.h>
 #include <qpainter.h>
 #include <qpaintengine.h>
+#include <qmath.h>
+#if QT_VERSION >= 0x040400
+#include <qthread.h>
+#include <qfuture.h>
+#include <qtconcurrentrun.h>
+#endif
 #include <float.h>
 
 class QwtPlotRasterItem::PrivateData
@@ -29,6 +33,7 @@ public:
     }
 
     int alpha;
+
     QwtPlotRasterItem::PaintAttributes paintAttributes;
 
     struct ImageCache
@@ -185,7 +190,8 @@ static QImage qwtExpandImage(const QImage &image,
                         yy2 = sz.height();
                 }
 
-                const quint32 *line1 = (const quint32 *) image.scanLine( y1 );
+                const quint32 *line1 =
+                    reinterpret_cast<const quint32 *>( image.scanLine( y1 ) );
 
                 for ( int x1 = 0; x1 < w; x1++ )
                 {
@@ -216,12 +222,14 @@ static QImage qwtExpandImage(const QImage &image,
                     const quint32 rgb( line1[x1] );
                     for ( int y2 = yy1; y2 < yy2; y2++ )
                     {
-                        quint32 *line2 = ( quint32 *) expanded.scanLine( y2 );
-                        for ( int x2 = xx1; x2 < xx2; x2++ ) 
+                        quint32 *line2 = reinterpret_cast<quint32 *>(
+                            expanded.scanLine( y2 ) );
+
+                        for ( int x2 = xx1; x2 < xx2; x2++ )
                             line2[x2] = rgb;
-                    }       
-                }   
-            }   
+                    }
+                }
+            }
             break;
         }
         case 8:
@@ -232,26 +240,26 @@ static QImage qwtExpandImage(const QImage &image,
                 if ( y1 == 0 )
                 {
                     yy1 = 0;
-                }   
+                }
                 else
                 {
                     yy1 = qRound( y1 * ph - py0 );
                     if ( yy1 < 0 )
-                        yy1 = 0; 
-                }       
-                
+                        yy1 = 0;
+                }
+
                 int yy2;
                 if ( y1 == h - 1 )
                 {
                     yy2 = sz.height();
-                }   
+                }
                 else
                 {
                     yy2 = qRound( ( y1 + 1 ) * ph - py0 );
                     if ( yy2 > sz.height() )
                         yy2 = sz.height();
                 }
-    
+
                 const uchar *line1 = image.scanLine( y1 );
 
                 for ( int x1 = 0; x1 < w; x1++ )
@@ -284,19 +292,19 @@ static QImage qwtExpandImage(const QImage &image,
                     {
                         uchar *line2 = expanded.scanLine( y2 );
                         memset( line2 + xx1, line1[x1], xx2 - xx1 );
-                    }       
-                }   
+                    }
+                }
             }
             break;
         }
         default:
             expanded = image;
     }
-    
-    return expanded;
-}   
 
-static QRectF expandToPixels(const QRectF &rect, const QRectF &pixelRect)
+    return expanded;
+}
+
+static QRectF qwtExpandToPixels(const QRectF &rect, const QRectF &pixelRect)
 {
     const double pw = pixelRect.width();
     const double ph = pixelRect.height();
@@ -315,7 +323,7 @@ static QRectF expandToPixels(const QRectF &rect, const QRectF &pixelRect)
     return r;
 }
 
-static void transformMaps( const QTransform &tr,
+static void qwtTransformMaps( const QTransform &tr,
     const QwtScaleMap &xMap, const QwtScaleMap &yMap,
     QwtScaleMap &xxMap, QwtScaleMap &yyMap )
 {
@@ -329,7 +337,7 @@ static void transformMaps( const QTransform &tr,
     yyMap.setPaintInterval( p1.y(), p2.y() );
 }
 
-static void adjustMaps( QwtScaleMap &xMap, QwtScaleMap &yMap,
+static void qwtAdjustMaps( QwtScaleMap &xMap, QwtScaleMap &yMap,
     const QRectF &area, const QRectF &paintRect)
 {
     double sx1 = area.left();
@@ -350,7 +358,7 @@ static void adjustMaps( QwtScaleMap &xMap, QwtScaleMap &yMap,
     yMap.setScaleInterval(sy1, sy2);
 }
 
-static bool useCache( QwtPlotRasterItem::CachePolicy policy,
+static bool qwtUseCache( QwtPlotRasterItem::CachePolicy policy,
     const QPainter *painter )
 {
     bool doCache = false;
@@ -376,39 +384,37 @@ static bool useCache( QwtPlotRasterItem::CachePolicy policy,
     return doCache;
 }
 
-static QImage toRgba( const QImage& image, int alpha )
+static void qwtToRgba( const QImage* from, QImage* to,
+    const QRect& tile, int alpha )
 {
-    if ( alpha < 0 || alpha >= 255 )
-        return image;
-
-    QImage alphaImage( image.size(), QImage::Format_ARGB32 );
-
     const QRgb mask1 = qRgba( 0, 0, 0, alpha );
     const QRgb mask2 = qRgba( 255, 255, 255, 0 );
     const QRgb mask3 = qRgba( 0, 0, 0, 255 );
 
-    const int w = image.size().width();
-    const int h = image.size().height();
+    const int y0 = tile.top();
+    const int y1 = tile.bottom();
+    const int x0 = tile.left();
+    const int x1 = tile.right();
 
-    if ( image.depth() == 8 )
+    if ( from->depth() == 8 )
     {
-        for ( int y = 0; y < h; y++ )
+        for ( int y = y0; y <= y1; y++ )
         {
-            QRgb* alphaLine = ( QRgb* )alphaImage.scanLine( y );
-            const unsigned char *line = image.scanLine( y );
+            QRgb *alphaLine = reinterpret_cast<QRgb *>( to->scanLine( y ) );
+            const unsigned char *line = from->scanLine( y );
 
-            for ( int x = 0; x < w; x++ )
-                *alphaLine++ = ( image.color( *line++ ) & mask2 ) | mask1;
+            for ( int x = x0; x <= x1; x++ )
+                *alphaLine++ = ( from->color( *line++ ) & mask2 ) | mask1;
         }
     }
-    else if ( image.depth() == 32 )
+    else if ( from->depth() == 32 )
     {
-        for ( int y = 0; y < h; y++ )
+        for ( int y = y0; y <= y1; y++ )
         {
-            QRgb* alphaLine = ( QRgb* )alphaImage.scanLine( y );
-            const QRgb* line = ( const QRgb* ) image.scanLine( y );
+            QRgb *alphaLine = reinterpret_cast<QRgb *>( to->scanLine( y ) );
+            const QRgb *line = reinterpret_cast<const QRgb *>( from->scanLine( y ) );
 
-            for ( int x = 0; x < w; x++ )
+            for ( int x = x0; x <= x1; x++ )
             {
                 const QRgb rgb = *line++;
                 if ( rgb & mask3 ) // alpha != 0
@@ -418,8 +424,6 @@ static QImage toRgba( const QImage& image, int alpha )
             }
         }
     }
-
-    return alphaImage;
 }
 
 //! Constructor
@@ -468,7 +472,7 @@ void QwtPlotRasterItem::setPaintAttribute( PaintAttribute attribute, bool on )
 }
 
 /*!
-    \brief Return the current paint attributes
+    \return True, when attribute is enabled
     \sa PaintAttribute, setPaintAttribute()
 */
 bool QwtPlotRasterItem::testPaintAttribute( PaintAttribute attribute ) const
@@ -568,16 +572,16 @@ void QwtPlotRasterItem::invalidateCache()
    \brief Pixel hint
 
    The geometry of a pixel is used to calculated the resolution and
-   alignment of the rendered image. 
+   alignment of the rendered image.
 
-   Width and height of the hint need to be the horizontal  
-   and vertical distances between 2 neighboured points. 
-   The center of the hint has to be the position of any point 
+   Width and height of the hint need to be the horizontal
+   and vertical distances between 2 neighbored points.
+   The center of the hint has to be the position of any point
    ( it doesn't matter which one ).
 
    Limiting the resolution of the image might significantly improve
    the performance and heavily reduce the amount of memory when rendering
-   a QImage from the raster data. 
+   a QImage from the raster data.
 
    The default implementation returns an empty rectangle (QRectF()),
    meaning, that the image will be rendered in target device ( f.e screen )
@@ -601,7 +605,7 @@ QRectF QwtPlotRasterItem::pixelHint( const QRectF &area ) const
   \param painter Painter
   \param xMap X-Scale Map
   \param yMap Y-Scale Map
-  \param canvasRect Contents rect of the plot canvas
+  \param canvasRect Contents rectangle of the plot canvas
 */
 void QwtPlotRasterItem::draw( QPainter *painter,
     const QwtScaleMap &xMap, const QwtScaleMap &yMap,
@@ -610,19 +614,19 @@ void QwtPlotRasterItem::draw( QPainter *painter,
     if ( canvasRect.isEmpty() || d_data->alpha == 0 )
         return;
 
-    const bool doCache = useCache( d_data->cache.policy, painter );
+    const bool doCache = qwtUseCache( d_data->cache.policy, painter );
 
     const QwtInterval xInterval = interval( Qt::XAxis );
     const QwtInterval yInterval = interval( Qt::YAxis );
 
     /*
-        Scaling a rastered image always results in a loss of
+        Scaling an image always results in a loss of
         precision/quality. So we always render the image in
         paint device resolution.
     */
 
     QwtScaleMap xxMap, yyMap;
-    transformMaps( painter->transform(), xMap, yMap, xxMap, yyMap );
+    qwtTransformMaps( painter->transform(), xMap, yMap, xxMap, yyMap );
 
     QRectF paintRect = painter->transform().mapRect( canvasRect );
     QRectF area = QwtScaleMap::invTransform( xxMap, yyMap, paintRect );
@@ -643,11 +647,11 @@ void QwtPlotRasterItem::draw( QPainter *painter,
     QRectF pixelRect = pixelHint(area);
     if ( !pixelRect.isEmpty() )
     {
-        const QRectF r = QwtScaleMap::invTransform( 
-            xxMap, yyMap, QRectF(0, 0, 1, 1) ).normalized();
+        // one pixel of the target device in plot coordinates
+        const double dx = qAbs( xxMap.invTransform( 1 ) - xxMap.invTransform( 0 ) );
+        const double dy = qAbs( yyMap.invTransform( 1 ) - yyMap.invTransform( 0 ) );
 
-        if ( r.width() > pixelRect.width() &&
-            r.height() > pixelRect.height() )
+        if ( dx > pixelRect.width() && dy > pixelRect.height() )
         {
             /*
               When the resolution of the data pixels is higher than
@@ -656,6 +660,19 @@ void QwtPlotRasterItem::draw( QPainter *painter,
              */
             pixelRect = QRectF();
         }
+        else
+        {
+            /*
+              If only one dimension is of the data pixel is higher
+              we expand the pixel rect to the resolution of the target device.
+             */
+
+            if ( dx > pixelRect.width() )
+                pixelRect.setWidth( dx );
+
+            if ( dy > pixelRect.height() )
+                pixelRect.setHeight( dy );
+        }
     }
 
     if ( pixelRect.isEmpty() )
@@ -666,13 +683,13 @@ void QwtPlotRasterItem::draw( QPainter *painter,
             // the aligned paint rectangle exactly match the area
 
             paintRect = qwtAlignRect(paintRect);
-            adjustMaps(xxMap, yyMap, area, paintRect);
+            qwtAdjustMaps(xxMap, yyMap, area, paintRect);
         }
 
         // When we have no information about position and size of
         // data pixels we render in resolution of the paint device.
 
-        image = compose(xxMap, yyMap, 
+        image = compose(xxMap, yyMap,
             area, paintRect, paintRect.size().toSize(), doCache);
         if ( image.isNull() )
             return;
@@ -680,19 +697,19 @@ void QwtPlotRasterItem::draw( QPainter *painter,
         // Remove pixels at the boundaries, when explicitly
         // excluded in the intervals
 
-        imageRect = qwtStripRect(paintRect, area, 
+        imageRect = qwtStripRect(paintRect, area,
             xxMap, yyMap, xInterval, yInterval);
 
         if ( imageRect != paintRect )
         {
-            const QRect r( 
+            const QRect r(
                 qRound( imageRect.x() - paintRect.x()),
                 qRound( imageRect.y() - paintRect.y() ),
                 qRound( imageRect.width() ),
                 qRound( imageRect.height() ) );
-                
+
             image = image.copy(r);
-        }   
+        }
     }
     else
     {
@@ -700,7 +717,7 @@ void QwtPlotRasterItem::draw( QPainter *painter,
             paintRect = qwtAlignRect(paintRect);
 
         // align the area to the data pixels
-        QRectF imageArea = expandToPixels(area, pixelRect);
+        QRectF imageArea = qwtExpandToPixels(area, pixelRect);
 
         if ( imageArea.right() == xInterval.maxValue() &&
             !( xInterval.borderFlags() & QwtInterval::ExcludeMaximum ) )
@@ -716,29 +733,31 @@ void QwtPlotRasterItem::draw( QPainter *painter,
         QSize imageSize;
         imageSize.setWidth( qRound( imageArea.width() / pixelRect.width() ) );
         imageSize.setHeight( qRound( imageArea.height() / pixelRect.height() ) );
-        image = compose(xxMap, yyMap, 
+
+        image = compose(xxMap, yyMap,
             imageArea, paintRect, imageSize, doCache );
+
         if ( image.isNull() )
             return;
 
-        imageRect = qwtStripRect(paintRect, area, 
+        imageRect = qwtStripRect(paintRect, area,
             xxMap, yyMap, xInterval, yInterval);
 
         if ( ( image.width() > 1 || image.height() > 1 ) &&
             testPaintAttribute( PaintInDeviceResolution ) )
         {
-            // Because of rounding errors the pixels 
-            // need to be expanded manually to rectangles of 
+            // Because of rounding errors the pixels
+            // need to be expanded manually to rectangles of
             // different sizes
 
-            image = qwtExpandImage(image, xxMap, yyMap, 
+            image = qwtExpandImage(image, xxMap, yyMap,
                 imageArea, area, paintRect, xInterval, yInterval );
         }
     }
 
     painter->save();
     painter->setWorldTransform( QTransform() );
-    
+
     QwtPainter::drawImage( painter, imageRect, image );
 
     painter->restore();
@@ -749,7 +768,7 @@ void QwtPlotRasterItem::draw( QPainter *painter,
 
    This method is intended to be reimplemented by derived classes.
    The default implementation returns an invalid interval.
-   
+
    \param axis X, Y, or Z axis
 */
 QwtInterval QwtPlotRasterItem::interval(Qt::Axis axis) const
@@ -759,7 +778,7 @@ QwtInterval QwtPlotRasterItem::interval(Qt::Axis axis) const
 }
 
 /*!
-   \return Bounding rect of the data
+   \return Bounding rectangle of the data
    \sa QwtPlotRasterItem::interval()
 */
 QRectF QwtPlotRasterItem::boundingRect() const
@@ -797,9 +816,9 @@ QRectF QwtPlotRasterItem::boundingRect() const
     return r.normalized();
 }
 
-QImage QwtPlotRasterItem::compose( 
+QImage QwtPlotRasterItem::compose(
     const QwtScaleMap &xMap, const QwtScaleMap &yMap,
-    const QRectF &imageArea, const QRectF &paintRect, 
+    const QRectF &imageArea, const QRectF &paintRect,
     const QSize &imageSize, bool doCache) const
 {
     QImage image;
@@ -822,14 +841,14 @@ QImage QwtPlotRasterItem::compose(
         if ( paintRect.toRect().width() > imageSize.width() )
             dx = imageArea.width() / imageSize.width();
 
-        const QwtScaleMap xxMap = 
+        const QwtScaleMap xxMap =
             imageMap(Qt::Horizontal, xMap, imageArea, imageSize, dx);
-        
+
         double dy = 0.0;
         if ( paintRect.toRect().height() > imageSize.height() )
             dy = imageArea.height() / imageSize.height();
 
-        const QwtScaleMap yyMap = 
+        const QwtScaleMap yyMap =
             imageMap(Qt::Vertical, yMap, imageArea, imageSize, dy);
 
         image = renderImage( xxMap, yyMap, imageArea, imageSize );
@@ -843,7 +862,43 @@ QImage QwtPlotRasterItem::compose(
     }
 
     if ( d_data->alpha >= 0 && d_data->alpha < 255 )
-        image = toRgba( image, d_data->alpha );
+    {
+        QImage alphaImage( image.size(), QImage::Format_ARGB32 );
+
+#if QT_VERSION >= 0x040400 && !defined(QT_NO_QFUTURE)
+        uint numThreads = renderThreadCount();
+
+        if ( numThreads <= 0 )
+            numThreads = QThread::idealThreadCount();
+
+        if ( numThreads <= 0 )
+            numThreads = 1;
+
+        const int numRows = image.height() / numThreads;
+
+        QList< QFuture<void> > futures;
+        for ( uint i = 0; i < numThreads; i++ )
+        {
+            QRect tile( 0, i * numRows, image.width(), numRows );
+            if ( i == numThreads - 1 )
+            {
+                tile.setHeight( image.height() - i * numRows );
+                qwtToRgba( &image, &alphaImage, tile, d_data->alpha );
+            }
+            else
+            {
+                futures += QtConcurrent::run(
+                    &qwtToRgba, &image, &alphaImage, tile, d_data->alpha );
+            }
+        }
+        for ( int i = 0; i < futures.size(); i++ )
+            futures[i].waitForFinished();
+#else
+        const QRect tile( 0, 0, image.width(), image.height() );
+        qwtToRgba( &image, &alphaImage, tile, d_data->alpha );
+#endif
+        image = alphaImage;
+    }
 
     return image;
 }
@@ -856,6 +911,8 @@ QImage QwtPlotRasterItem::compose(
    \param area Area to be painted on the image
    \param imageSize Image size
    \param pixelSize Width/Height of a data pixel
+
+   \return Calculated scale map
 */
 QwtScaleMap QwtPlotRasterItem::imageMap(
     Qt::Orientation orientation,
