@@ -23,18 +23,25 @@
 #include <qtconcurrentrun.h>
 #endif
 
+#define DEBUG_RENDER 0
+
+#if DEBUG_RENDER
+#include <QElapsedTimer>
+#endif
+
 class QwtPlotSpectrogram::PrivateData
 {
 public:
     PrivateData():
-        data( NULL ),
-        renderThreadCount( 1 )
+        data( NULL )
     {
         colorMap = new QwtLinearColorMap();
         displayMode = ImageMode;
 
         conrecFlags = QwtRasterData::IgnoreAllVerticesOnLevel;
+#if 0
         conrecFlags |= QwtRasterData::IgnoreOutOfRange;
+#endif
     }
     ~PrivateData()
     {
@@ -46,8 +53,6 @@ public:
     QwtColorMap *colorMap;
     DisplayModes displayMode;
 
-    uint renderThreadCount;
-
     QList<double> contourLevels;
     QPen defaultContourPen;
     QwtRasterData::ConrecFlags conrecFlags;
@@ -107,6 +112,7 @@ void QwtPlotSpectrogram::setDisplayMode( DisplayMode mode, bool on )
             d_data->displayMode &= ~mode;
     }
 
+    legendChanged();
     itemChanged();
 }
 
@@ -121,37 +127,6 @@ bool QwtPlotSpectrogram::testDisplayMode( DisplayMode mode ) const
     return ( d_data->displayMode & mode );
 }
 
-/*!
-   Rendering an image from the raster data can often be done
-   parallel on a multicore system.
-
-   \param numThreads Number of threads to be used for rendering.
-                     If numThreads is set to 0, the system specific
-                     ideal thread count is used.
-
-   The default thread count is 1 ( = no additional threads )
-
-   \warning Rendering in multiple threads is only supported for Qt >= 4.4
-   \sa renderThreadCount(), renderImage(), renderTile()
-*/
-void QwtPlotSpectrogram::setRenderThreadCount( uint numThreads )
-{
-    d_data->renderThreadCount = numThreads;
-}
-
-/*!
-   \return Number of threads to be used for rendering.
-           If numThreads is set to 0, the system specific
-           ideal thread count is used.
-
-   \warning Rendering in multiple threads is only supported for Qt >= 4.4
-   \sa setRenderThreadCount(), renderImage(), renderTile()
-*/
-uint QwtPlotSpectrogram::renderThreadCount() const
-{
-    return d_data->renderThreadCount;
-}
-
 /*!
   Change the color map
 
@@ -172,6 +147,8 @@ void QwtPlotSpectrogram::setColorMap( QwtColorMap *colorMap )
     }
 
     invalidateCache();
+
+    legendChanged();
     itemChanged();
 }
 
@@ -184,6 +161,25 @@ const QwtColorMap *QwtPlotSpectrogram::colorMap() const
     return d_data->colorMap;
 }
 
+/*!
+  Build and assign the default pen for the contour lines
+
+  In Qt5 the default pen width is 1.0 ( 0.0 in Qt4 ) what makes it
+  non cosmetic ( see QPen::isCosmetic() ). This method has been introduced
+  to hide this incompatibility.
+
+  \param color Pen color
+  \param width Pen width
+  \param style Pen style
+
+  \sa pen(), brush()
+ */
+void QwtPlotSpectrogram::setDefaultContourPen(
+    const QColor &color, qreal width, Qt::PenStyle style )
+{
+    setDefaultContourPen( QPen( color, width, style ) );
+}
+
 /*!
    \brief Set the default pen for the contour lines
 
@@ -199,6 +195,8 @@ void QwtPlotSpectrogram::setDefaultContourPen( const QPen &pen )
     if ( pen != d_data->defaultContourPen )
     {
         d_data->defaultContourPen = pen;
+
+        legendChanged();
         itemChanged();
     }
 }
@@ -265,6 +263,8 @@ void QwtPlotSpectrogram::setConrecFlag(
    \param flag CONREC flag
    \return true, is enabled
 
+   The default setting enables QwtRasterData::IgnoreAllVerticesOnLevel
+
    \sa setConrecClag(), renderContourLines(),
        QwtRasterData::contourLines()
 */
@@ -287,11 +287,13 @@ void QwtPlotSpectrogram::setContourLevels( const QList<double> &levels )
 {
     d_data->contourLevels = levels;
     qSort( d_data->contourLevels );
+
+    legendChanged();
     itemChanged();
 }
 
 /*!
-   \brief Return the levels of the contour lines.
+   \return Levels of the contour lines.
 
    The levels are sorted in increasing order.
 
@@ -360,7 +362,7 @@ QwtInterval QwtPlotSpectrogram::interval(Qt::Axis axis) const
    \brief Pixel hint
 
    The geometry of a pixel is used to calculated the resolution and
-   alignment of the rendered image. 
+   alignment of the rendered image.
 
    The default implementation returns data()->pixelHint( rect );
 
@@ -369,7 +371,7 @@ QwtInterval QwtPlotSpectrogram::interval(Qt::Axis axis) const
 
    \return Bounding rectangle of a pixel
 
-   \sa QwtPlotRasterItem::pixelHint(), QwtRasterData::pixelHint(), 
+   \sa QwtPlotRasterItem::pixelHint(), QwtRasterData::pixelHint(),
        render(), renderImage()
 */
 QRectF QwtPlotSpectrogram::pixelHint( const QRectF &area ) const
@@ -383,7 +385,7 @@ QRectF QwtPlotSpectrogram::pixelHint( const QRectF &area ) const
 /*!
    \brief Render an image from data and color map.
 
-   For each pixel of rect the value is mapped into a color.
+   For each pixel of area the value is mapped into a color.
 
   \param xMap X-Scale Map
   \param yMap Y-Scale Map
@@ -400,7 +402,7 @@ QImage QwtPlotSpectrogram::renderImage(
     const QwtScaleMap &xMap, const QwtScaleMap &yMap,
     const QRectF &area, const QSize &imageSize ) const
 {
-    if ( imageSize.isEmpty() || d_data->data == NULL 
+    if ( imageSize.isEmpty() || d_data->data == NULL
         || d_data->colorMap == NULL )
     {
         return QImage();
@@ -420,8 +422,13 @@ QImage QwtPlotSpectrogram::renderImage(
 
     d_data->data->initRaster( area, image.size() );
 
+#if DEBUG_RENDER
+    QElapsedTimer time;
+    time.start();
+#endif
+
 #if QT_VERSION >= 0x040400 && !defined(QT_NO_QFUTURE)
-    uint numThreads = d_data->renderThreadCount;
+    uint numThreads = renderThreadCount();
 
     if ( numThreads <= 0 )
         numThreads = QThread::idealThreadCount();
@@ -455,6 +462,11 @@ QImage QwtPlotSpectrogram::renderImage(
     renderTile( xMap, yMap, tile, &image );
 #endif
 
+#if DEBUG_RENDER
+    const qint64 elapsed = time.elapsed();
+    qDebug() << "renderImage" << imageSize << elapsed;
+#endif
+
     d_data->data->discardRaster();
 
     return image;
@@ -485,7 +497,7 @@ void QwtPlotSpectrogram::renderTile(
         {
             const double ty = yMap.invTransform( y );
 
-            QRgb *line = ( QRgb * )image->scanLine( y );
+            QRgb *line = reinterpret_cast<QRgb *>( image->scanLine( y ) );
             line += tile.left();
 
             for ( int x = tile.left(); x <= tile.right(); x++ )
@@ -520,21 +532,21 @@ void QwtPlotSpectrogram::renderTile(
 /*!
    \brief Return the raster to be used by the CONREC contour algorithm.
 
-   A larger size will improve the precisision of the CONREC algorithm,
+   A larger size will improve the precision of the CONREC algorithm,
    but will slow down the time that is needed to calculate the lines.
 
    The default implementation returns rect.size() / 2 bounded to
    the resolution depending on pixelSize().
 
-   \param area Rect, where to calculate the contour lines
-   \param rect Rect in pixel coordinates, where to paint the contour lines
+   \param area Rectangle, where to calculate the contour lines
+   \param rect Rectangle in pixel coordinates, where to paint the contour lines
    \return Raster to be used by the CONREC contour algorithm.
 
    \note The size will be bounded to rect.size().
 
    \sa drawContourLines(), QwtRasterData::contourLines()
 */
-QSize QwtPlotSpectrogram::contourRasterSize( 
+QSize QwtPlotSpectrogram::contourRasterSize(
     const QRectF &area, const QRect &rect ) const
 {
     QSize raster = rect.size() / 2;
@@ -555,6 +567,7 @@ QSize QwtPlotSpectrogram::contourRasterSize(
 
    \param rect Rectangle, where to calculate the contour lines
    \param raster Raster, used by the CONREC algorithm
+   \return Calculated contour lines
 
    \sa contourLevels(), setConrecFlag(),
        QwtRasterData::contourLines()
@@ -586,9 +599,7 @@ void QwtPlotSpectrogram::drawContourLines( QPainter *painter,
     if ( d_data->data == NULL )
         return;
 
-    const QwtInterval intensityRange = d_data->data->interval( Qt::ZAxis );
-
-    const int numLevels = ( int )d_data->contourLevels.size();
+    const int numLevels = d_data->contourLevels.size();
     for ( int l = 0; l < numLevels; l++ )
     {
         const double level = d_data->contourLevels[l];
@@ -603,7 +614,7 @@ void QwtPlotSpectrogram::drawContourLines( QPainter *painter,
         painter->setPen( pen );
 
         const QPolygonF &lines = contourLines[level];
-        for ( int i = 0; i < ( int )lines.size(); i += 2 )
+        for ( int i = 0; i < lines.size(); i += 2 )
         {
             const QPointF p1( xMap.transform( lines[i].x() ),
                 yMap.transform( lines[i].y() ) );
@@ -621,7 +632,7 @@ void QwtPlotSpectrogram::drawContourLines( QPainter *painter,
   \param painter Painter
   \param xMap Maps x-values into pixel coordinates.
   \param yMap Maps y-values into pixel coordinates.
-  \param canvasRect Contents rect of the canvas in painter coordinates
+  \param canvasRect Contents rectangle of the canvas in painter coordinates
 
   \sa setDisplayMode(), renderImage(),
       QwtPlotRasterItem::draw(), drawContourLines()
