@@ -16,6 +16,11 @@
 #include <qpainter.h>
 #include <qmath.h>
 
+#if QT_VERSION < 0x040601
+#define qFastSin(x) qSin(x)
+#define qFastCos(x) qCos(x)
+#endif
+
 class QwtScaleDraw::PrivateData
 {
 public:
@@ -58,6 +63,7 @@ QwtScaleDraw::~QwtScaleDraw()
 /*!
    Return alignment of the scale
    \sa setAlignment()
+   \return Alignment of the scale
 */
 QwtScaleDraw::Alignment QwtScaleDraw::alignment() const
 {
@@ -67,6 +73,8 @@ QwtScaleDraw::Alignment QwtScaleDraw::alignment() const
 /*!
    Set the alignment of the scale
 
+   \param align Alignment of the scale
+
    The default alignment is QwtScaleDraw::BottomScale
    \sa alignment()
 */
@@ -81,6 +89,8 @@ void QwtScaleDraw::setAlignment( Alignment align )
   TopScale, BottomScale are horizontal (Qt::Horizontal) scales,
   LeftScale, RightScale are vertical (Qt::Vertical) scales.
 
+  \return Orientation of the scale
+
   \sa alignment()
 */
 Qt::Orientation QwtScaleDraw::orientation() const
@@ -107,11 +117,11 @@ Qt::Orientation QwtScaleDraw::orientation() const
   \param start Start border distance
   \param end End border distance
 */
-void QwtScaleDraw::getBorderDistHint( const QFont &font,
-                                      int &start, int &end ) const
+void QwtScaleDraw::getBorderDistHint(
+    const QFont &font, int &start, int &end ) const
 {
     start = 0;
-    end = 0;
+    end = 1.0;
 
     if ( !hasComponent( QwtAbstractScaleDraw::Labels ) )
         return;
@@ -129,7 +139,7 @@ void QwtScaleDraw::getBorderDistHint( const QFont &font,
     double maxTick = minTick;
     double maxPos = minPos;
 
-    for ( uint i = 1; i < ( uint )ticks.count(); i++ )
+    for ( int i = 1; i < ticks.count(); i++ )
     {
         const double tickPos = scaleMap().transform( ticks[i] );
         if ( tickPos < minPos )
@@ -188,7 +198,7 @@ int QwtScaleDraw::minLabelDist( const QFont &font ) const
         return 0;
 
     const QList<double> &ticks = scaleDiv().ticks( QwtScaleDiv::MajorTick );
-    if ( ticks.count() == 0 )
+    if ( ticks.isEmpty() )
         return 0;
 
     const QFontMetrics fm( font );
@@ -199,21 +209,22 @@ int QwtScaleDraw::minLabelDist( const QFont &font ) const
     QRectF bRect2 = labelRect( font, ticks[0] );
     if ( vertical )
     {
-        bRect2.setRect( -bRect2.bottom(), 0, bRect2.height(), bRect2.width() );
+        bRect2.setRect( -bRect2.bottom(), 0.0, bRect2.height(), bRect2.width() );
     }
-    int maxDist = 0;
 
-    for ( uint i = 1; i < ( uint )ticks.count(); i++ )
+    double maxDist = 0.0;
+
+    for ( int i = 1; i < ticks.count(); i++ )
     {
         bRect1 = bRect2;
         bRect2 = labelRect( font, ticks[i] );
         if ( vertical )
         {
-            bRect2.setRect( -bRect2.bottom(), 0,
+            bRect2.setRect( -bRect2.bottom(), 0.0,
                 bRect2.height(), bRect2.width() );
         }
 
-        int dist = fm.leading(); // space between the labels
+        double dist = fm.leading(); // space between the labels
         if ( bRect1.right() > 0 )
             dist += bRect1.right();
         if ( bRect2.left() < 0 )
@@ -223,26 +234,24 @@ int QwtScaleDraw::minLabelDist( const QFont &font ) const
             maxDist = dist;
     }
 
-    double angle = labelRotation() / 180.0 * M_PI;
+    double angle = qwtRadians( labelRotation() );
     if ( vertical )
         angle += M_PI / 2;
 
-    if ( qSin( angle ) == 0.0 )
-        return maxDist;
+    const double sinA = qFastSin( angle ); // qreal -> double
+    if ( qFuzzyCompare( sinA + 1.0, 1.0 ) )
+        return qCeil( maxDist );
 
     const int fmHeight = fm.ascent() - 2;
 
     // The distance we need until there is
     // the height of the label font. This height is needed
-    // for the neighbour labal.
+    // for the neighbored label.
 
-    int labelDist = ( int )( fmHeight / qSin( angle ) * qCos( angle ) );
+    double labelDist = fmHeight / qFastSin( angle ) * qFastCos( angle );
     if ( labelDist < 0 )
         labelDist = -labelDist;
 
-    // The cast above floored labelDist. We want to ceil.
-    labelDist++;
-
     // For text orientations close to the scale orientation
 
     if ( labelDist > maxDist )
@@ -254,7 +263,7 @@ int QwtScaleDraw::minLabelDist( const QFont &font ) const
     if ( labelDist < fmHeight )
         labelDist = fmHeight;
 
-    return labelDist;
+    return qCeil( labelDist );
 }
 
 /*!
@@ -266,6 +275,7 @@ int QwtScaleDraw::minLabelDist( const QFont &font ) const
    of the labels.
 
    \param font Font used for painting the labels
+   \return Extent
 
    \sa minLength()
 */
@@ -291,7 +301,7 @@ double QwtScaleDraw::extent( const QFont &font ) const
 
     if ( hasComponent( QwtAbstractScaleDraw::Backbone ) )
     {
-        const double pw = qMax( 1, penWidth() );  // penwidth can be zero
+        const double pw = qMax( 1, penWidth() );  // pen width can be zero
         d += pw;
     }
 
@@ -303,6 +313,7 @@ double QwtScaleDraw::extent( const QFont &font ) const
    Calculate the minimum length that is needed to draw the scale
 
    \param font Font used for painting the labels
+   \return Minimum length that is needed to draw the scale
 
    \sa extent()
 */
@@ -336,10 +347,11 @@ int QwtScaleDraw::minLength( const QFont &font ) const
 /*!
    Find the position, where to paint a label
 
-   The position has a distance of majTickLength() + spacing() + 1
-   from the backbone. The direction depends on the alignment()
+   The position has a distance that depends on the length of the ticks
+   in direction of the alignment().
 
    \param value Value
+   \return Position, where to paint a label
 */
 QPointF QwtScaleDraw::labelPosition( double value ) const
 {
@@ -390,7 +402,7 @@ QPointF QwtScaleDraw::labelPosition( double value ) const
 
    \param painter Painter
    \param value Value of the tick
-   \param len Lenght of the tick
+   \param len Length of the tick
 
    \sa drawBackbone(), drawLabel()
 */
@@ -596,14 +608,22 @@ QPointF QwtScaleDraw::pos() const
   The length doesn't include the space needed for
   overlapping labels.
 
+  \param length Length of the backbone
+
   \sa move(), minLabelDist()
 */
 void QwtScaleDraw::setLength( double length )
 {
+#if 1
     if ( length >= 0 && length < 10 )
         length = 10;
+
+    // why should we accept negative lengths ???
     if ( length < 0 && length > -10 )
         length = -10;
+#else
+    length = qMax( length, 10 );
+#endif
 
     d_data->len = length;
     updateMap();
@@ -647,13 +667,15 @@ void QwtScaleDraw::drawLabel( QPainter *painter, double value ) const
 }
 
 /*!
-  Find the bounding rect for the label. The coordinates of
-  the rect are absolute coordinates ( calculated from pos() ).
+  \brief Find the bounding rectangle for the label.
+
+  The coordinates of the rectangle are absolute ( calculated from pos() ).
   in direction of the tick.
 
   \param font Font used for painting
   \param value Value
 
+  \return Bounding rectangle
   \sa labelRect()
 */
 QRect QwtScaleDraw::boundingLabelRect( const QFont &font, double value ) const
@@ -676,6 +698,7 @@ QRect QwtScaleDraw::boundingLabelRect( const QFont &font, double value ) const
    \param pos Position where to paint the label
    \param size Size of the label
 
+   \return Transformation matrix
    \sa setLabelAlignment(), setLabelRotation()
 */
 QTransform QwtScaleDraw::labelTransformation(
@@ -739,12 +762,14 @@ QTransform QwtScaleDraw::labelTransformation(
 }
 
 /*!
-  Find the bounding rect for the label. The coordinates of
-  the rect are relative to spacing + ticklength from the backbone
+  Find the bounding rectangle for the label. The coordinates of
+  the rectangle are relative to spacing + tick length from the backbone
   in direction of the tick.
 
   \param font Font used for painting
   \param value Value
+
+   \return Bounding rectangle that is needed to draw a label
 */
 QRectF QwtScaleDraw::labelRect( const QFont &font, double value ) const
 {
@@ -768,6 +793,8 @@ QRectF QwtScaleDraw::labelRect( const QFont &font, double value ) const
 
    \param font Label font
    \param value Value
+
+   \return Size that is needed to draw a label
 */
 QSizeF QwtScaleDraw::labelSize( const QFont &font, double value ) const
 {
@@ -804,7 +831,7 @@ double QwtScaleDraw::labelRotation() const
 /*!
   \brief Change the label flags
 
-  Labels are aligned to the point ticklength + spacing away from the backbone.
+  Labels are aligned to the point tick length + spacing away from the backbone.
 
   The alignment is relative to the orientation of the label text.
   In case of an flags of 0 the label will be aligned
@@ -817,13 +844,13 @@ double QwtScaleDraw::labelRotation() const
 
   Changing the alignment is often necessary for rotated labels.
 
-  \param alignment Or'd Qt::AlignmentFlags <see qnamespace.h>
+  \param alignment Or'd Qt::AlignmentFlags see <qnamespace.h>
 
   \sa setLabelRotation(), labelRotation(), labelAlignment()
   \warning The various alignments might be confusing.
            The alignment of the label is not the alignment
            of the scale and is not the alignment of the flags
-           (QwtText::flags()) returned from QwtAbstractScaleDraw::label().
+           ( QwtText::flags() ) returned from QwtAbstractScaleDraw::label().
 */
 
 void QwtScaleDraw::setLabelAlignment( Qt::Alignment alignment )
@@ -846,21 +873,21 @@ Qt::Alignment QwtScaleDraw::labelAlignment() const
 */
 int QwtScaleDraw::maxLabelWidth( const QFont &font ) const
 {
-    int maxWidth = 0;
+    double maxWidth = 0.0;
 
     const QList<double> &ticks = scaleDiv().ticks( QwtScaleDiv::MajorTick );
-    for ( uint i = 0; i < ( uint )ticks.count(); i++ )
+    for ( int i = 0; i < ticks.count(); i++ )
     {
         const double v = ticks[i];
         if ( scaleDiv().contains( v ) )
         {
-            const int w = labelSize( font, ticks[i] ).width();
+            const double w = labelSize( font, ticks[i] ).width();
             if ( w > maxWidth )
                 maxWidth = w;
         }
     }
 
-    return maxWidth;
+    return qCeil( maxWidth );
 }
 
 /*!
@@ -869,21 +896,21 @@ int QwtScaleDraw::maxLabelWidth( const QFont &font ) const
 */
 int QwtScaleDraw::maxLabelHeight( const QFont &font ) const
 {
-    int maxHeight = 0;
+    double maxHeight = 0.0;
 
     const QList<double> &ticks = scaleDiv().ticks( QwtScaleDiv::MajorTick );
-    for ( uint i = 0; i < ( uint )ticks.count(); i++ )
+    for ( int i = 0; i < ticks.count(); i++ )
     {
         const double v = ticks[i];
         if ( scaleDiv().contains( v ) )
         {
-            const int h = labelSize( font, ticks[i] ).height();
+            const double h = labelSize( font, ticks[i] ).height();
             if ( h > maxHeight )
                 maxHeight = h;
         }
     }
 
-    return maxHeight;
+    return qCeil( maxHeight );
 }
 
 void QwtScaleDraw::updateMap()
